/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console;

import java.awt.Dimension;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.awt.event.InputEvent;
import java.awt.event.KeyEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.util.Iterator;

import javax.swing.JFrame;
import javax.swing.JTabbedPane;
import javax.swing.KeyStroke;
import javax.swing.SwingUtilities;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import net.sf.dc.console.components.DcFrame;
import net.sf.dc.console.components.DcPanel;
import net.sf.dc.console.components.panels.ModuleListPanel;
import net.sf.dc.console.components.renderers.DcTableHeaderRenderer;
import net.sf.dc.console.components.renderers.DcTableHeaderRendererRequired;
import net.sf.dc.console.components.renderers.DcTableRowIndexCellRenderer;
import net.sf.dc.console.dialogs.AboutDialog;
import net.sf.dc.console.forms.CategoryForm;
import net.sf.dc.console.forms.StateForm;
import net.sf.dc.console.forms.StorageMediaForm;
import net.sf.dc.console.menu.DcQuickFilterToolBar;
import net.sf.dc.console.menu.DcToolBar;
import net.sf.dc.console.note.NotePanel;
import net.sf.dc.console.software.SoftwarePlatformForm;
import net.sf.dc.console.views.MasterView;
import net.sf.dc.console.views.View;
import net.sf.dc.console.views.list.ListSearchView;
import net.sf.dc.console.views.table.TableSearchView;
import net.sf.dc.core.IconLibrary;
import net.sf.dc.core.Repository;
import net.sf.dc.core.actions.ChangeViewAction;
import net.sf.dc.core.actions.CreateNewAction;
import net.sf.dc.core.actions.CreateReportAction;
import net.sf.dc.core.actions.DeleteAction;
import net.sf.dc.core.actions.ImportAction;
import net.sf.dc.core.actions.InternetUpdateAction;
import net.sf.dc.core.actions.LogAction;
import net.sf.dc.core.actions.NewItemWizardAction;
import net.sf.dc.core.actions.OpenAction;
import net.sf.dc.core.actions.OpenFieldsSettingsDialog;
import net.sf.dc.core.actions.OpenModuleAction;
import net.sf.dc.core.actions.OpenResourceEditorAction;
import net.sf.dc.core.actions.SaveAllAction;
import net.sf.dc.core.actions.SearchAction;
import net.sf.dc.core.actions.SearchAgainAction;
import net.sf.dc.core.actions.SettingsAction;
import net.sf.dc.core.actions.StartProgramAction;
import net.sf.dc.core.actions.ToggleModuleListAction;
import net.sf.dc.core.actions.ToggleQuickFilterBar;
import net.sf.dc.core.actions.ToggleQuickViewAction;
import net.sf.dc.core.actions.ToggleToolbarLabelsAction;
import net.sf.dc.core.data.DataFilters;
import net.sf.dc.core.data.DataManager;
import net.sf.dc.core.modules.DcCategoryModule;
import net.sf.dc.core.modules.DcModule;
import net.sf.dc.core.modules.DcModules;
import net.sf.dc.core.modules.DcStateModule;
import net.sf.dc.core.modules.DcStorageMediaModule;
import net.sf.dc.core.modules.software.SoftwarePlatformModule;
import net.sf.dc.core.modules.template.ITemplateModule;
import net.sf.dc.core.objects.DcLookAndFeel;
import net.sf.dc.db.DatabaseManager;
import net.sf.dc.helpsystem.JavaHelp;
import net.sf.dc.messages.Messages;
import net.sf.dc.messages.MessagesWriter;
import net.sf.dc.resources.DcResources;
import net.sf.dc.settings.DcSettings;
import net.sf.dc.settings.ReportSettings;
import net.sf.dc.settings.definitions.DcFieldDefinitions;
import net.sf.ext.jflexsett.settings.ui.SettingsView;

public class MainFrame extends DcFrame {

	private static final long serialVersionUID = 9L;
	public static final int _SEARCHTAB = 0;
    public static final int _INSERTTAB = 1;

    private ModuleListPanel moduleListPanel = new ModuleListPanel();
    private JTabbedPane tabbedPane = ComponentFactory.getTabbedPane();
    private final NotePanel notePanel = new NotePanel();
    private SettingsView settingsView;

    private DcToolBar toolBar;
    private DcQuickFilterToolBar quickFilterTb;

    private DcModule activeModule = null;

    private boolean initialized = false;

    public MainFrame() {
        super(Repository.system.product, IconLibrary._icoMain, null);

        setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
        this.getContentPane().setLayout(Layout.getGBL());

        toggleModuleList(DcSettings.getValueAsBoolean(Repository.settings.stShowModuleList));
        tabbedPane.getModel().addChangeListener(new TabIndexListener());

        try {
            addWindowListener(new WindowAdapter() {
                public void windowClosing(WindowEvent e) {
                    finish();
                }
            });

            changeModule(DcSettings.getValueAsInt(Repository.settings.stModule), false);

            getContentPane().add(moduleListPanel, Layout.getGBC( 0, 1, 1, 1, 1.0, 1.0
                                ,GridBagConstraints.NORTHWEST, GridBagConstraints.VERTICAL,
                                 new Insets(0, 0, 0, 0), 0, 0));
            getContentPane().add(tabbedPane, Layout.getGBC( 1, 1, 1, 1, 50.0, 50.0
                                 ,GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                                 new Insets(0, 0, 0, 0), 0, 0));

            tabbedPane.setPreferredSize(DcSettings.getValueAsDimension(Repository.settings.stTabbedPaneSize));
            pack();
            tabbedPane.setSize(DcSettings.getValueAsDimension(Repository.settings.stTabbedPaneSize));

        } catch (Error err) {
            Messages.add(err);
        }

        updateLAF(DcSettings.getValueAsLookAndFeel(Repository.settings.stLookAndFeel));

        setResizable(true);
        setSize(DcSettings.getValueAsDimension(Repository.settings.stMainViewSize));

        registerShortCuts();
        
        setCenteredLocation();

        initialized = true;
    }
    
    private void registerShortCuts() {
        addKeyListener(KeyStroke.getKeyStroke(KeyEvent.VK_D, InputEvent.CTRL_MASK), new DeleteAction(), "delete");
        addKeyListener(KeyStroke.getKeyStroke(KeyEvent.VK_E, InputEvent.CTRL_MASK), new CreateReportAction(), "report");
        addKeyListener(KeyStroke.getKeyStroke(KeyEvent.VK_F, InputEvent.CTRL_MASK), new SearchAction(), "find");
        addKeyListener(KeyStroke.getKeyStroke(KeyEvent.VK_G, InputEvent.CTRL_MASK), new SearchAgainAction(), "findagain");
        addKeyListener(KeyStroke.getKeyStroke(KeyEvent.VK_I, InputEvent.CTRL_MASK), new ImportAction(), "fileimport");
        addKeyListener(KeyStroke.getKeyStroke(KeyEvent.VK_L, InputEvent.CTRL_MASK), new LogAction(), "showlog");
        addKeyListener(KeyStroke.getKeyStroke(KeyEvent.VK_M, InputEvent.CTRL_MASK), new SettingsAction(), "settings");
        addKeyListener(KeyStroke.getKeyStroke(KeyEvent.VK_N, InputEvent.CTRL_MASK), new CreateNewAction(), "createnew");
        addKeyListener(KeyStroke.getKeyStroke(KeyEvent.VK_O, InputEvent.CTRL_MASK), new OpenAction(), "open");
        addKeyListener(KeyStroke.getKeyStroke(KeyEvent.VK_P, InputEvent.CTRL_MASK), new StartProgramAction(), "startProgram");
        addKeyListener(KeyStroke.getKeyStroke(KeyEvent.VK_R, InputEvent.CTRL_MASK), new OpenResourceEditorAction(), "resourceeditor");
        addKeyListener(KeyStroke.getKeyStroke(KeyEvent.VK_S, InputEvent.CTRL_MASK), new SaveAllAction(), "saveall");
        addKeyListener(KeyStroke.getKeyStroke(KeyEvent.VK_T, InputEvent.CTRL_MASK), new OpenFieldsSettingsDialog(), "fieldsettings");
        addKeyListener(KeyStroke.getKeyStroke(KeyEvent.VK_U, InputEvent.CTRL_MASK), new InternetUpdateAction(), "internetupdateaction");
        addKeyListener(KeyStroke.getKeyStroke(KeyEvent.VK_W, InputEvent.CTRL_MASK), new NewItemWizardAction(), "wizard");
        
        addKeyListener(KeyStroke.getKeyStroke("F2"), new ChangeViewAction(MasterView._TABLE_VIEW), "opentableview");
        addKeyListener(KeyStroke.getKeyStroke("F3"), new ChangeViewAction(MasterView._LIST_VIEW), "openlistview");
        
        addKeyListener(KeyStroke.getKeyStroke("F7"), new ToggleQuickFilterBar(), "togglequickfilterbar");
        addKeyListener(KeyStroke.getKeyStroke("F8"), new ToggleModuleListAction(), "togglemodules");
        addKeyListener(KeyStroke.getKeyStroke("F9"), new ToggleQuickViewAction(), "togglequickview");
        addKeyListener(KeyStroke.getKeyStroke("F10"), new ToggleToolbarLabelsAction(), "toggletoolbarlabels");;

        for (Iterator iter = DcModules.getModules(); iter.hasNext(); ) {
            DcModule module = (DcModule) iter.next();
            if (module.isTopModule() && module.getKeyStroke() != null)
                addKeyListener(module.getKeyStroke(), new OpenModuleAction(module.getIndex()), "open" + module.getName() + "module");
        }        
    }
    
    public void setVisible(boolean b) {
        super.setVisible(b);
    }

    public DcToolBar getToolBar() {
        return toolBar;
    }

    public void setSelectedTab(int index) {
    	tabbedPane.setSelectedIndex(index);
    }

    public int getSelectedTab() {
        return tabbedPane.getSelectedIndex();
    }

    public void showSettingsView() {
        settingsView = DcSettings.getView(this);
    	settingsView.setVisible(true);
        settingsView.dispose();
        settingsView = null;
    }

    public void showAboutDialog() {
        new AboutDialog(this).setVisible(true);
    }

    public NotePanel getNotePanel() {
    	return notePanel;
    }

    private void toggleModuleList(boolean enable) {
        moduleListPanel.setVisible(enable);
    }

    public void toggleModuleList() {
    	moduleListPanel.setVisible(!moduleListPanel.isVisible());
        DcSettings.setValue(Repository.settings.stShowModuleList, new Boolean(moduleListPanel.isVisible()));
    }

    public void updateQuickFilterBar() {
        boolean show = DcSettings.getValueAsBoolean(Repository.settings.stShowQuickFilterBar);
        
        if (quickFilterTb != null)
            getContentPane().remove(quickFilterTb);
        
        if (show) {
            quickFilterTb = new DcQuickFilterToolBar(activeModule);
            getContentPane().add(quickFilterTb, Layout.getGBC( 0, 1, 2, 1, 1.0, 1.0
                    ,GridBagConstraints.NORTHWEST, GridBagConstraints.VERTICAL,
                     new Insets(0, 0, 0, 0), 0, 0));     
            
            quickFilterTb.revalidate();
        }
        
        try {
        	pack();
        } catch (Exception exp) {}
    }
    
    private void applySettings(DcModule module) {
        Font fontNormal = DcSettings.getValueAsFont(Repository.settings.stSystemFontNormal);
        Font fontSystem = DcSettings.getValueAsFont(Repository.settings.stSystemFontBold);
        
        MessagesWriter.logForm.setFont(fontNormal);
        
        tabbedPane.setFont(fontSystem);
        notePanel.setFont(fontNormal);
        
        remove(moduleListPanel);
        moduleListPanel = new ModuleListPanel();
        getContentPane().add(moduleListPanel, Layout.getGBC( 0, 2, 1, 1, 1.0, 1.0
                            ,GridBagConstraints.NORTHWEST, GridBagConstraints.VERTICAL,
                             new Insets(0, 0, 0, 0), 0, 0));
        
        moduleListPanel.setFont(fontSystem);
        moduleListPanel.setSelectedModule(DcModules.getCurrentModule().getIndex());
        moduleListPanel.revalidate();
        
        moduleListPanel.setVisible(DcSettings.getValueAsBoolean(Repository.settings.stShowModuleList));
        
        if (settingsView != null)
            settingsView.setFont(fontNormal);
        
        if (module.getMenuBar() != null)
            module.getMenuBar().setFont(fontSystem);
        
        if (toolBar != null) {
            toolBar.setFloatable(false);
            toolBar.setFont(fontSystem);
        }
        
        if (initialized) {
            if (module.isTopModule() && module.getSearchForm() != null) 
                module.getSearchForm().setFont(fontNormal);
        }
           
        if (module.getIndex() == DcModules._SOFTWARE) {
            SoftwarePlatformModule m = (SoftwarePlatformModule) DcModules.getModule(DcModules._SOFTWAREPLATFORM);
            SoftwarePlatformForm form = m.getForm();
            form.setFont(fontNormal);
        }
        
        if (module.getStateModule() != null) {
            StateForm form = module.getStateModule().getForm();
            form.setFont(fontNormal);
        }

        if (module.getStorageMediaModule() != null) {
            StorageMediaForm form = module.getStorageMediaModule().getForm();
            form.setFont(fontNormal);
        }        
        
        if (module.getCategoryModule() != null) {
            CategoryForm form = module.getCategoryModule().getForm();
            form.setFont(fontNormal);
        }
        
        String key = module.getFieldDefinitionsSettingsKey();
        if (key != null) {
            DcFieldDefinitions definitions = (DcFieldDefinitions) DcSettings.getValue(key);
            module.getDcObject().applySettings(definitions);

            if (module.getSearchView() != null)
                module.getSearchView().applySettings();

            if (module.getInsertView() != null)
                module.getInsertView().applySettings();
        }
    }
    
    public void updateLAF(DcLookAndFeel laf) {
        DcSettings.setValue(Repository.settings.stLookAndFeel, laf);
        
        ComponentFactory.setLookAndFeel();

        tabbedPane.removeAll();
        SwingUtilities.updateComponentTreeUI(moduleListPanel);
        SwingUtilities.updateComponentTreeUI(tabbedPane);
        SwingUtilities.updateComponentTreeUI(toolBar);
        
        if (quickFilterTb != null)
            SwingUtilities.updateComponentTreeUI(quickFilterTb);

        SwingUtilities.updateComponentTreeUI(notePanel);
        SwingUtilities.updateComponentTreeUI(MessagesWriter.logForm.getContentPane());
        
        if (initialized)
            SwingUtilities.updateComponentTreeUI(DcModules.getCurrentModule().getSearchForm().getContentPane());
        
        SwingUtilities.updateComponentTreeUI(DcTableHeaderRenderer.button);
        SwingUtilities.updateComponentTreeUI(DcTableHeaderRendererRequired.button);
        SwingUtilities.updateComponentTreeUI(DcTableRowIndexCellRenderer.button);
        
        for (Iterator iter = DcModules.getModules(); iter.hasNext(); ) {
            DcModule module = (DcModule) iter.next();
            
            if (module.getMenuBar() != null)
                SwingUtilities.updateComponentTreeUI(module.getMenuBar());
            
            if (initialized) {
                if (module.isTopModule() && module.getSearchForm() != null) 
                    SwingUtilities.updateComponentTreeUI(module.getSearchForm().getContentPane());
            }

            if (module instanceof SoftwarePlatformModule)
                SwingUtilities.updateComponentTreeUI(((SoftwarePlatformModule) module).getForm().getContentPane());

            if (module instanceof DcStorageMediaModule)
                SwingUtilities.updateComponentTreeUI(((DcStorageMediaModule) module).getForm().getContentPane());
            
            if (module instanceof DcStateModule)
                SwingUtilities.updateComponentTreeUI(((DcStateModule) module).getForm().getContentPane());

            if (module instanceof ITemplateModule)
                SwingUtilities.updateComponentTreeUI(((ITemplateModule) module).getForm().getContentPane());
            
            if (module instanceof DcCategoryModule) {
                if (((DcCategoryModule) module).getForm() != null)
                    SwingUtilities.updateComponentTreeUI(((DcCategoryModule) module).getForm().getContentPane());
            }
            
            MasterView[] views = module.getViews();
            if (views != null) {
                for (int i = 0; i < views.length; i++)  {
                    MasterView view = views[i];
                    if (view.get(MasterView._LIST_VIEW) != null)
                        SwingUtilities.updateComponentTreeUI(view.get(MasterView._LIST_VIEW));
                    if (view.get(MasterView._TABLE_VIEW) != null)
                        SwingUtilities.updateComponentTreeUI(view.get(MasterView._TABLE_VIEW));
                }
            }
        }

        changeModule(DcModules.getCurrentModule().getIndex(), true);
        
        applySettings();        
        
        validate();
    }    
    
    public void applySettings() {
        for (Iterator iter = DcModules.getModules(); iter.hasNext(); ) {
            DcModule module = (DcModule) iter.next();
            
            if (module.isTopModule())
                applySettings(module);    
        }
    }

    public void applyView(int newView) {
        int previousView = DcSettings.getValueAsInt(Repository.settings.stSelectedView);
        if (previousView != newView) {
            DcSettings.setValue(Repository.settings.stSelectedView, new Integer(newView));
            tabbedPane.removeAll();
            MasterView[] masterViews = activeModule.getViews();
            for (int i = 0; i < masterViews.length; i++) {
                View view = masterViews[i].getCurrent();
                if (view != null) {
                    tabbedPane.addTab(view.getTitle(), view.getIcon(), view);
                    
                    if (view instanceof TableSearchView || view instanceof ListSearchView)
                    	DataManager.bindData(view, activeModule.getIndex(), 
                    					 DataFilters.getCurrent(activeModule.getIndex()), false);
                }
            }
            tabbedPane.addTab(DcResources.getText("lblTabNote"), IconLibrary._icoNote, notePanel);
        }
    }
    
    public void updateMenuBar() {
        setJMenuBar(null);
        setJMenuBar(activeModule.getMenuBar());
        repaint();
        validate();
    }

    public void changeModule(int index, boolean loadData) {
        activeModule = DcModules.getModule(index);
        activeModule = activeModule == null ? DcModules.getModule(DcModules._SOFTWARE) : activeModule;
        DcSettings.setValue(Repository.settings.stModule, new Integer(activeModule.getIndex()));

        if (initialized && activeModule.getQuickView() != null)
            activeModule.getQuickView().changeSize();

        moduleListPanel.setSelectedModule(index);
        
        updateMenuBar();

        if (toolBar != null)
            getContentPane().remove(toolBar);

        toolBar = new DcToolBar(activeModule);
        getContentPane().add(toolBar, Layout.getGBC( 0, 0, 2, 1, 1.0, 1.0
                            ,GridBagConstraints.NORTHWEST, GridBagConstraints.VERTICAL,
                             new Insets(0, 0, 0, 0), 0, 0));
        
        updateQuickFilterBar();

        Dimension size = tabbedPane.getSize();
        if (size.height == 0 || size.width == 0)
        	size = DcSettings.getValueAsDimension(Repository.settings.stMainViewSize);

        getContentPane().remove(tabbedPane);
        tabbedPane.removeAll();
        tabbedPane = null;
        tabbedPane = ComponentFactory.getTabbedPane();
        tabbedPane.setPreferredSize(size);
        tabbedPane.addChangeListener(new TabIndexListener());
        
        getContentPane().add(tabbedPane, Layout.getGBC( 1, 2, 1, 1, 50.0, 50.0
                            ,GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                             new Insets(0, 0, 0, 0), 0, 0));

        try {
            MasterView[] masterViews = activeModule.getViews();
            for (int i = 0; i < masterViews.length; i++) {
                View view = masterViews[i].getCurrent();
                if (view != null) {
                    tabbedPane.addTab(view.getTitle(), view.getIcon(), view);
                    
                    if (loadData && (view instanceof ListSearchView || view instanceof TableSearchView))
                        DataManager.bindData(view, activeModule.getIndex(), DataFilters.getCurrent(activeModule.getIndex()), false);
                }
            }
            
            tabbedPane.addTab(DcResources.getText("lblTabNote"), IconLibrary._icoNote, notePanel);
            
        } catch (Exception exp) {}
        
        getContentPane().setBackground(activeModule.getMenuBar().getBackground());
        repaint();
    }

    public void finish() {
        Messages.add(DcResources.getText("msgApplicationStops"), Messages._STANDARD);

        setVisible(false);

        DatabaseManager.closeDatabases(false);

        DcSettings.setValue(Repository.settings.stTabbedPaneSize, tabbedPane.getSize());
        DcSettings.setValue(Repository.settings.stMainViewSize, getSize());

        DcSettings.save();
        ReportSettings.save();

        System.exit(0);
    }

    private class TabIndexListener implements ChangeListener {
        public void stateChanged(ChangeEvent arg0) {
            int index = tabbedPane.getSelectedIndex();
            if (index > -1) {
            	DcPanel panel = (DcPanel) tabbedPane.getComponent(index);
            	JavaHelp.setHelpIndex(panel.getHelpIndex());
            }
	    }
    }
}