/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console.components;

import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.Insets;
import java.awt.RenderingHints;
import java.awt.Toolkit;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.io.File;
import java.lang.ref.SoftReference;
import java.net.URL;

import javax.swing.JComponent;

import net.sf.dc.console.forms.PictureView;
import net.sf.dc.core.IconLibrary;
import net.sf.dc.messages.Messages;
import net.sf.dc.resources.DcResources;
import net.sf.ext.jflexsett.settings.ui.ISettingsComponent;
import net.sf.ext.util.Base64;
import net.sf.ext.util.Utilities;

public class DcPictureField extends JComponent implements ISettingsComponent {

    byte[] bytes = null;
    private Image img = null;

    private boolean scaled = true;
    private boolean thumbnail = false;
    private Dimension size = null;
    
    private Insets insets = new Insets(0, 0, 0, 0);

    private static final Toolkit tk = Toolkit.getDefaultToolkit();
    
    public DcPictureField(boolean allowZoom) {
        this(true, allowZoom, false);
    }

    public DcPictureField(boolean scaled, boolean allowZoom, boolean thumbnail) {
        this.scaled = scaled;
        this.thumbnail = thumbnail;
        if (allowZoom) {
            setToolTipText(DcResources.getText("tpPictureField"));
            addMouseListener(new OpenPictureFrameAction());
        }
    }

    public void setValue(Object o) {
        flushImage();

        try {
            if (o == null) {
                bytes = null;
            } else {
                if (o instanceof URL) {
                    URL url = (URL) o;
                    bytes = Utilities.readFile(new File(url.getFile()));
                } else if (o instanceof String) {
                    String value = (String) o;
                    if (value.endsWith("jpg")) {
                        URL url;
                        if (!value.startsWith("file:"))
                        	url = new URL("file:" + value);
                        else
                            url = new URL(value);

                        bytes = Utilities.readFile(new File(url.getFile()));
                    } else {
                        String base64 = (String) o;
                        if (base64 != null && base64.length() > 0)
                            bytes = Base64.decode(base64.toCharArray());
                    }
                } else if (o instanceof byte[]) {
                    bytes = (byte[]) o;
                }
            }
        } catch (Exception exp) {
            Messages.add(DcResources.getText("msgCouldNotLoadPicture"), Messages._ERROR);
            Messages.add(exp);
        }

        bytes = bytes == null || bytes.length == 0 ? null : bytes; 
        if (bytes != null)
            img = tk.createImage(bytes);

        new SoftReference(img);
        new SoftReference(bytes);
        
        revalidate();
        repaint();
    }

    public void flush() {
        flushImage();
    }

    private void flushImage() {
        bytes = null;
        size = null;
        img = null;
        size = null;
        insets = null;        
    }

    public boolean isEmpty() {
    	return img == null;
    }

    public Object getValue() {
        return bytes;
    }

    private Image getImage() {
    	return img;
    }

    private boolean scalingAllowed(int width, int height) {
        return scaled && (height >= 50 && width >= 50);
    }

    public void paint(Graphics g) {
        insets = getInsets(insets);
        size = getSize(size);
        
        Image image = getImage();
        int w;
        int h;
        if (image == null) {
            image = IconLibrary._icoEmptyPicture;
            scaled = true;
        }

        if (image != null && scalingAllowed(image.getWidth(null), image.getHeight(null))) {
            int imageWidth = image.getWidth(null);
            int imageHeight = image.getHeight(null);

            if (g instanceof Graphics2D)
                ((Graphics2D) g).setRenderingHint(
                        RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
            
            w = 0;
            h = 0;
            if (thumbnail) {
                w =  Math.min(size.width, image.getWidth(this) / 5);
                h = Math.min(size.height, image.getHeight(this) / 5);
            } else {
                w =  Math.min(size.width, image.getWidth(this));
                h = Math.min(size.height, image.getHeight(this));

                double scaledRatio = (double) w / (double) h;
                double imageRatio = (double) imageWidth / (double) imageHeight;
                if (scaledRatio < imageRatio) 
                    h = (int)(w / imageRatio);
                else
                    w = (int)(h * imageRatio);
            }
        } else {
            w = image.getWidth(null);
            h = image.getHeight(null);
        }

        g.drawImage(image, getInset(size.width, w), getInset(size.height, h), w, h, null);
        g.dispose();
    }

    private int getInset(int sizeComp, int sizePic) {
        int remainingSpace = sizeComp - sizePic;
        return remainingSpace >= 2 ? remainingSpace / 2 : 0;
    }

    public Dimension getMinimumSize() {
        int imgw = 32, imgh = 32;
        Image image = getImage();
        if (thumbnail && image != null) {
            imgw =  image.getWidth(this)/ 5;
            imgh = image.getHeight(this)/ 5;
        } else if (image != null) {
            imgw = image.getWidth(this);
            imgh = image.getHeight(this);
        }
        insets = getInsets(insets);
        return new Dimension(
            insets.left + Math.max(32, imgw / 10) + insets.right,
            insets.top + Math.max(32, imgh / 10) + insets.bottom);
    }

    public Dimension getPreferredSize() {
        int imgw = 32, imgh = 32;
        Image image = getImage();
        if (thumbnail && image != null) {
            imgw =  image.getWidth(this)/ 5;
            imgh = image.getHeight(this)/ 5;
        } else if (image != null) {
            imgw = image.getWidth(this);
            imgh = image.getHeight(this);
        }

        insets = getInsets(insets);
        return new Dimension(
            insets.left + imgw + insets.right,
            insets.top + imgh + insets.bottom);
    }

    public Dimension getMaximumSize() {
        return getPreferredSize();
    }

    public void showFullPicture() {
        PictureView view = new PictureView(bytes);
        if (!view.wasClosed()) {
        	setValue(view.getValue());
        }
    }
    
    public class OpenPictureFrameAction implements MouseListener {
        public void mouseReleased(MouseEvent e) {
            if (e.getClickCount() == 2) {
                showFullPicture();
            }
        }
        public void mouseEntered(MouseEvent e) {}
        public void mouseExited(MouseEvent e) {}
        public void mousePressed(MouseEvent e) {}
        public void mouseClicked(MouseEvent e) {}
    }
    
    protected void paintComponent(Graphics g) {
        Graphics2D g2d = (Graphics2D) g;
        g2d.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
        super.paintComponent(g2d);
    }     
}