package net.sf.dc.console.forms;

import java.awt.Color;
import java.awt.Component;
import java.awt.Cursor;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.util.Collection;

import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JLayeredPane;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JRootPane;
import javax.swing.JScrollPane;
import javax.swing.JTabbedPane;
import javax.swing.JViewport;
import javax.swing.SwingUtilities;

import net.sf.dc.console.ComponentFactory;
import net.sf.dc.console.Layout;
import net.sf.dc.console.MainFrame;
import net.sf.dc.console.components.DcFrame;
import net.sf.dc.console.components.DcPanel;
import net.sf.dc.console.components.lists.DcObjectList;
import net.sf.dc.console.components.lists.DcObjectListElement;
import net.sf.dc.console.dialogs.MessageBox;
import net.sf.dc.console.dialogs.QuestionBox;
import net.sf.dc.console.menu.DcSimpleItemViewPopupMenu;
import net.sf.dc.core.IconLibrary;
import net.sf.dc.core.Repository;
import net.sf.dc.core.modules.DcModule;
import net.sf.dc.core.modules.DcModules;
import net.sf.dc.core.objects.DcObject;
import net.sf.dc.processes.DataTask;
import net.sf.dc.resources.DcResources;
import net.sf.dc.settings.DcSettings;
import net.sf.dc.wf.requests.IRequest;
import net.sf.dc.wf.requests.RequestCollection;
import net.sf.dc.wf.requests.StatusUpdateRequest;
import net.sf.ext.util.SwingUtil;

public abstract class DcSimpleItemView extends DcFrame {

    protected DataTask task;
    
    protected final ListMouseListener mouseListener = new ListMouseListener();
    private final boolean readonly; 
    
    private JScrollPane scroller;
    
    private final JPanel panelActions = new JPanel();
    private JPanel statusPanel;
    private final JButton buttonNew = ComponentFactory.getButton(DcResources.getText("lblNew"));
    private final JButton buttonCancel = ComponentFactory.getButton(DcResources.getText("lblCancel"));
    
    
    protected final DcObjectList list;
    
    private final int module;
    private final DcPanel panel = new DcPanel("", null);
    
    public DcSimpleItemView(JFrame parent, int module, boolean readonly) {
        super(DcModules.getModule(module).getObjectNamePlural(), IconLibrary._icoAnchor, parent);
        
        this.list = new DcObjectList(DcModules.getModule(module), false, false);
        this.readonly = readonly;
        
        SwingUtil.setRootFrame(this);
        this.module = module;
        addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                close();
            }
        });        
        
        buildPanel();

        pack();
        
        setSize(DcSettings.getValueAsDimension(getSizeSettingsKey()));
        setCenteredLocation();
    }
    
    protected abstract String getSizeSettingsKey();
    public abstract void open();
    public abstract void addObjects();
    protected abstract void close();
    public abstract void createNew();
    public abstract RequestCollection getAfterDeleteRequests();
    
    public void setObjects(Collection objects) {
        list.clear();
        list.add(objects);
    }
    
    public Collection getObjects() {
        return list.getObjects();
    }
    
    protected JPopupMenu getPopupMenu() {
        return new DcSimpleItemViewPopupMenu(list.getSelectedObject(), this);
    }
    
    protected int getModuleIdx() {
        return module;
    }
    
    protected DcModule getModule() {
        return DcModules.getModule(module);
    }
    
    public void setVisible(boolean b) {
        super.setVisible(b);
        if (b)
            addObjects();
    }    
    
    public void setBackground(Color color) {
        super.setBackground(color);
        getContentPane().setBackground(color);
        Component[] components = getComponents();
        for (int i = 0; i < components.length; i++) {
            JComponent component = (JComponent) components[i];
            setBackground(component, color);
        }
    }

    private void setBackground(JComponent component, Color color) {
        if (component instanceof JLabel || component instanceof JPanel ||
            component instanceof JCheckBox || component instanceof JTabbedPane ||
            component instanceof JRootPane || component instanceof JLayeredPane) {

            component.setBackground(color);

            Component[] components = component.getComponents();
            for (int i = 0; i < components.length; i++) {
                JComponent subComponent = (JComponent) components[i];
                setBackground(subComponent, color);
            }
        }
    }

    public void setFont(Font font) {
        Font fontNormal = DcSettings.getValueAsFont(Repository.settings.stSystemFontNormal);
        Font fontSystem = DcSettings.getValueAsFont(Repository.settings.stSystemFontBold);

        super.setFont(fontNormal);
        if (panel != null) {
            panel.setFont(fontSystem);
            buttonCancel.setFont(fontSystem);
            buttonNew.setFont(fontSystem);
            list.setFont(fontNormal);
        }
    }      
    
    public void denyActions() {
        list.removeMouseListener(mouseListener);    
    }

    public void allowActions() {
        list.addMouseListener(mouseListener);
    }    
    
    /**
     * Indicates whether there is a data task running at this moment
     */
    protected boolean isTaskRunning() {
        boolean isTaskRunning = task != null && task.isRunning();
        if (isTaskRunning)
            new MessageBox(DcResources.getText("msgJobRunning"), MessageBox._WARNING); 
        return isTaskRunning; 
    }     

    private void buildPanel() {
        getContentPane().setLayout(Layout.getGBL());
        
        scroller = new JScrollPane(list);
        scroller.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
        scroller.getViewport().setScrollMode(JViewport.SIMPLE_SCROLL_MODE);        
        
        list.addMouseListener(mouseListener);
        
        //**********************************************************
        //Result panel
        //**********************************************************
        panel.setLayout(Layout.getGBL());
        panel.add(scroller,  Layout.getGBC( 0, 0, 1, 1, 10.0, 10.0
                ,GridBagConstraints.NORTH, GridBagConstraints.BOTH,
                 new Insets(5, 5, 5, 5), 0, 0));
        
        //**********************************************************
        //Action panel
        //**********************************************************
        panelActions.setLayout(new FlowLayout(FlowLayout.LEFT));
        buttonNew.addActionListener(new NewAction());
        buttonCancel.addActionListener(new CloseAction());
        
        buttonNew.setMnemonic('N');
        buttonCancel.setMnemonic('C');
        
        panelActions.add(buttonNew);
        panelActions.add(buttonCancel);
        
        //**********************************************************
        //Main panel
        //**********************************************************
        statusPanel = panel.getStatusPanel();
        
        getContentPane().add(panel,         Layout.getGBC( 0, 0, 1, 1, 10.0, 10.0
                ,GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                 new Insets(0, 0, 0, 0), 0, 0));
        
        if (!readonly) {
            getContentPane().add(panelActions,  Layout.getGBC( 0, 1, 1, 1, 1.0, 1.0
                    ,GridBagConstraints.NORTHEAST, GridBagConstraints.NONE,
                     new Insets(0, 0, 0, 0), 0, 0));
            getContentPane().add(statusPanel,  Layout.getGBC( 0, 2, 1, 1, 1.0, 1.0
                    ,GridBagConstraints.SOUTHWEST, GridBagConstraints.HORIZONTAL,
                     new Insets(0, 0, 0, 0), 0, 0));
        }
        
    }
    
    
    public void initProgressBar(int maxValue) {
        panel.initProgressBar(maxValue);
    }
    
    public void updateProgressBar(int value) {
        panel.updateProgressBar(value);
    }
    
    public void setStatus(String text) {
        panel.setStatus(text);
    }    
    
    public void delete() {
        if (!isTaskRunning()) {
            int[] rows = list.getSelectedIndices();
            if (rows.length > 0) {
                task = new DeletingThread(rows);
                task.start();
            } else {
                new MessageBox(DcResources.getText("msgSelectItemToDel"), MessageBox._WARNING);    
            }
        }
    }     
    
    protected class DeletingThread extends DataTask {

        public DeletingThread(int[] rows) {
            super(rows);
        }
        
        public void run() {
            try {
                updateProgressBar(0);
                initProgressBar(rows.length);   
                list.setCursor(new Cursor(Cursor.WAIT_CURSOR));
                
                setStatus("Deleting " + rows.length + " items from to the database");
                startRunning();
                denyActions();
                
                new QuestionBox(DcResources.getText("msgDeleteQuestion")); 
                if (!QuestionBox.result.booleanValue()) {
                    stopRunning();
                }
    
                int counter = 1;
                for (int i = rows.length -1; i > -1 && keepOnRunning(); i--) {
                    updateProgressBar(counter++);
                    
                    DcObjectListElement element =  (DcObjectListElement) list.getModel().getElementAt(rows[i]);
                    DcObject dco = element.getDcObject();

                    dco.markAsUnchanged();
                    dco.setSilence(true);
                    dco.delete(); 
                    
                    if (i == 0) {
                        RequestCollection requests = getAfterDeleteRequests();
                        for (int j = 0; j < requests.getRequests().length; j++) {
                            dco.addRequest(requests.getRequests()[j]);
                        }
                        
                        dco.setEndOfBatch(true);
                        IRequest request = 
                            new StatusUpdateRequest(dco.getModule().getIndex(), MainFrame._SEARCHTAB, "msgDeleteSuccessfull");
                        request.setExecuteOnFail(true);
                        dco.addRequest(request);
                    } else {
                        dco.setEndOfBatch(false);
                    }
                    
                    try {
                        sleep(300);
                    } catch (Exception ignore) {} // doesn't want to sleep..
                }
                
                list.setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
            } finally {
                list.setSelectedIndex(0);
                stopRunning();
                allowActions();
            }
        }
    }     
    
    /**
     * Listens on right clicks: shows the popup menu 
     * 
     * Listens on double clicks:
     * opens the item form for the row on which is double clicked
     */
    private class ListMouseListener implements MouseListener {
        public void mouseReleased(MouseEvent e) {
            if (!readonly) {
                if (SwingUtilities.isRightMouseButton(e)) {
                    if (list.getSelectedIndex() == -1) {
                        int index = list.locationToIndex(e.getPoint());
                        list.setSelectedIndex(index);
                    }
                    
                    if (list.getSelectedIndex() > -1) {
                        JPopupMenu menu = getPopupMenu();                
                        menu.setInvoker(list);
                        menu.show(list, e.getX(), e.getY());
                    }
                }
    
                if (e.getClickCount() == 2 && list.getSelectedIndex() > -1) 
                    open();
            }
        }
        public void mouseEntered(MouseEvent e) {}
        public void mouseExited(MouseEvent e) {}
        public void mousePressed(MouseEvent e) {}
        public void mouseClicked(MouseEvent e) {}
    }

    private class NewAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            createNew();
        }
    }
    
    private class CloseAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            close();
        }
    }    
}