/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console.reporting;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.File;
import java.util.Collection;

import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JPanel;
import javax.swing.JProgressBar;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;

import net.sf.dc.console.ComponentFactory;
import net.sf.dc.console.Layout;
import net.sf.dc.console.components.DcFileField;
import net.sf.dc.console.components.DcFrame;
import net.sf.dc.console.components.panels.ReportFieldSelectorPanel;
import net.sf.dc.console.dialogs.MessageBox;
import net.sf.dc.core.DataCrow;
import net.sf.dc.core.IconLibrary;
import net.sf.dc.core.Repository;
import net.sf.dc.messages.Messages;
import net.sf.dc.reporting.HtmlCardReport;
import net.sf.dc.reporting.HtmlListReport;
import net.sf.dc.reporting.PdfCardReport;
import net.sf.dc.reporting.PdfListReport;
import net.sf.dc.reporting.Report;
import net.sf.dc.reporting.TextReport;
import net.sf.dc.resources.DcResources;
import net.sf.dc.settings.DcSettings;
import net.sf.dc.settings.ReportSettings;
import net.sf.dc.settings.definitions.ReportFieldDefinitions;
import net.sf.ext.jflexsett.settings.ui.SettingsView;
import net.sf.ext.util.BrowserLauncher;
import net.sf.ext.util.FileFilter;

public class ReportingDialog extends DcFrame {

    private Report report;

    private String extension = "pdf";

    private JButton buttonRun = ComponentFactory.getButton(DcResources.getText("lblRun"));
    private JButton buttonStop = ComponentFactory.getButton(DcResources.getText("lblStop"));
    private JButton buttonCancel = ComponentFactory.getButton(DcResources.getText("lblCancel"));
    private JButton buttonResults = ComponentFactory.getButton(DcResources.getText("lblOpenReport"));
    private JButton buttonSettings = ComponentFactory.getButton(DcResources.getText("lblSettings"));

    private JComboBox comboReportType = new JComboBox();
    private ReportFieldSelectorPanel fieldTable;

    private DcFileField fileField;
    private JTextArea textLog = ComponentFactory.getTextArea();
    private JProgressBar progressBar = new JProgressBar();
    private Collection objects;

    public ReportingDialog(Collection objects) {
        super(DcResources.getText("lblCreateReport"), IconLibrary._icoReport, DataCrow.mainFrame);

        try {
            this.objects = objects;

            setHelpIndex("dc.Reporting");
    
            int module = DcSettings.getValueAsInt(Repository.settings.stModule);
            ReportSettings.setCurrentReport(ReportSettings._PDF_CARD_REPORT, module);
    
            fieldTable = new ReportFieldSelectorPanel();
            fileField = ComponentFactory.getFileField(true, false, null);
            buildDialog();
    
            addWindowListener(new WindowAdapter() {
                public void windowClosing(WindowEvent e) {
                    close();
                }
            });
        } catch (Exception exp) {
            new MessageBox(DcResources.getText("msgFailedToOpen", exp.getMessage()) , MessageBox._ERROR);
            Messages.add(exp);
        }
    }

    private void saveDefaults() {
        DcSettings.setValue(Repository.settings.stReportingDialogSize, getSize());
    }

    private void storeSettings() {
        int reportType = comboReportType.getSelectedIndex();
        ReportSettings.setValue(Repository.reportsettings.stReportType,
                                new Integer(reportType));
        ReportSettings.setValue(Repository.reportsettings.stTarget,
                                fileField.getFileAsString());
        ReportSettings.setValue(Repository.reportsettings.stFieldDefinitions,
                                fieldTable.getDefinitions());
        ReportSettings.save();
    }

    public void initProgressBar(int maxValue) {
        progressBar.setValue(0);
        progressBar.setMaximum(maxValue);
    }

    public void updateProgressBar(int value) {
        progressBar.setValue(value);
    }

    public void addMessage(String message) {
        textLog.insert(message + '\n', 0);
        textLog.setCaretPosition(0);
    }

    public void allowActions(boolean b) {
        buttonRun.setEnabled(b);
        buttonResults.setEnabled(b);
    }

    private void showResults() {
    	File file = fileField.getFile();
        if (file != null && file.exists()) {
            try {
            	BrowserLauncher.openURL(file.toString());
            } catch (Exception exp) {
                new MessageBox(DcResources.getText("msgErrorWhileOpeningX",
                               new String[] {file.toString(), exp.getMessage()}), MessageBox._WARNING);
                Messages.add(exp);
            }
        } else {
        	new MessageBox(DcResources.getText("msgCouldNotOpenReport"), MessageBox._WARNING);
        }
    }

    private void createReport() {
        storeSettings();

    	File file = fileField.getFile();
        ReportFieldDefinitions definitions = fieldTable.getDefinitions();

        if (file != null) {

            if (!file.getName().endsWith("." + extension)) {
                file.delete();
                file = new File(file.toString() + "." + extension);
                fileField.setFile(file);
            }

            int module = DcSettings.getValueAsInt(Repository.settings.stModule);
            int type = comboReportType.getSelectedIndex();

            if (type == 0)
                report = new PdfCardReport(this, objects, file, module, definitions);
            else if (type == 1)
                report = new HtmlCardReport(this, objects, file, module, definitions);
            else if (type == 2)
                report = new PdfListReport(this, objects, file, module, definitions);
            else if (type == 3)
                report = new HtmlListReport(this, objects, file, module, definitions);
            else if (type == 4)
                report = new TextReport(this, objects, file, module, definitions);

            allowActions(false);
            report.compile();
        } else {
            new MessageBox(DcResources.getText("msgSelectTargetFile"), MessageBox._WARNING);
        }
    }

    private void cancel() {
        if (report != null) {
            report.cancel();
        }
        allowActions(true);
    }

    private void close() {
        saveDefaults();
        cancel();
        storeSettings();
        
        report = null;
        extension = null;
        buttonRun = null;
        buttonStop = null;
        buttonCancel = null;
        buttonResults = null;
        buttonSettings = null;
        comboReportType = null;
        fieldTable = null;
        fileField = null;
        textLog = null;
        progressBar = null;
        
        if (objects != null) 
            objects.clear();
        
        objects = null;
        dispose();
    }

    private void buildDialog() {
        //**********************************************************
        //Input Panel
        //**********************************************************
        JPanel panelInput = new JPanel();
        panelInput.setLayout(Layout.getGBL());

        String target = ReportSettings.getValueAsString(Repository.reportsettings.stTarget);
        fileField.setValue(target);

        panelInput.add(fileField,  Layout.getGBC( 0, 0, 1, 1, 1.0, 1.0
                      ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                       new Insets( 5, 5, 5, 5), 0, 0));
        panelInput.setBorder(ComponentFactory.getTitleBorder(DcResources.getText("lblTargetFile")));

        //**********************************************************
        //Fields Panel
        //**********************************************************
        JPanel panelFields = new JPanel();
        panelFields.setLayout(Layout.getGBL());

        panelFields.add(fieldTable,  Layout.getGBC( 0, 0, 1, 1, 1.0, 1.0
                       ,GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                        new Insets( 5, 5, 5, 5), 0, 0));
        panelFields.setBorder(ComponentFactory.getTitleBorder(DcResources.getText("lblFields")));


        //**********************************************************
        //Report Type
        //**********************************************************
        JPanel panelReportType = new JPanel();
        panelReportType.setLayout(Layout.getGBL());

        comboReportType.addItem(DcResources.getText("lblPdfCardReport"));
        comboReportType.addItem(DcResources.getText("lblHtmlCardReport"));
        comboReportType.addItem(DcResources.getText("lblPdfListReport"));
        comboReportType.addItem(DcResources.getText("lblHtmlListReport"));
        comboReportType.addItem(DcResources.getText("lblTextReport"));
        comboReportType.addActionListener(new ReportTypeListener());

        int current = ReportSettings.getValueAsInt(Repository.reportsettings.stReportType);
        comboReportType.setSelectedIndex(current);

        panelReportType.add(comboReportType, Layout.getGBC( 0, 0, 1, 1, 1.0, 1.0
                       ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                        new Insets( 5, 5, 5, 5), 0, 0));
        panelReportType.setBorder(ComponentFactory.getTitleBorder(DcResources.getText("lblReportType")));

    	//**********************************************************
        //Actions Panel
        //**********************************************************
        JPanel panelActions = new JPanel();
        panelActions.setLayout(Layout.getGBL());

        buttonRun.addActionListener(new CreateReportAction());
        buttonStop.addActionListener(new CancelButtonAction());
        buttonResults.addActionListener(new ResultsButtonAction());
        buttonCancel.addActionListener(new CloseButtonAction());
        buttonSettings.addActionListener(new OpenSettingsViewAction());

        buttonRun.setMnemonic(KeyEvent.VK_R);
        buttonStop.setMnemonic(KeyEvent.VK_T);
        buttonCancel.setMnemonic(KeyEvent.VK_C);
        buttonResults.setMnemonic(KeyEvent.VK_O);

        panelActions.add(buttonRun,  Layout.getGBC( 0, 0, 1, 1, 1.0, 1.0
                ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                 new Insets( 5, 5, 5, 5), 0, 0));
        panelActions.add(buttonStop,    Layout.getGBC( 1, 0, 1, 1, 1.0, 1.0
                ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                 new Insets( 5, 5, 5, 5), 0, 0));
        panelActions.add(buttonResults,   Layout.getGBC( 2, 0, 1, 1, 1.0, 1.0
                ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                 new Insets( 5, 5, 5, 5), 0, 0));
        panelActions.add(buttonSettings,  Layout.getGBC( 3, 0, 1, 1, 1.0, 1.0
                ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                new Insets( 5, 5, 5, 5), 0, 0));        
        panelActions.add(buttonCancel,     Layout.getGBC( 4, 0, 1, 1, 1.0, 1.0
                ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                 new Insets( 5, 5, 5, 5), 0, 0));

        //**********************************************************
        //Progress panel
        //**********************************************************
        JPanel panelProgress = new JPanel();
        panelProgress.setLayout(Layout.getGBL());
        panelProgress.add(progressBar, Layout.getGBC( 0, 1, 1, 1, 1.0, 1.0
                         ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                          new Insets(5, 5, 5, 5), 0, 0));

        //**********************************************************
        //Log Panel
        //**********************************************************
        JPanel panelLog = new JPanel();
        panelLog.setLayout(Layout.getGBL());

        JScrollPane scroller = new JScrollPane(textLog);
        scroller.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
        panelLog.add(scroller, Layout.getGBC( 0, 1, 1, 1, 1.0, 1.0
                    ,GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                     new Insets(5, 5, 5, 5), 0, 0));

        panelLog.setBorder(ComponentFactory.getTitleBorder(DcResources.getText("lblLog")));

        //**********************************************************
        //Main Panel
        //**********************************************************
        this.getContentPane().setLayout(Layout.getGBL());
        this.getContentPane().add(
                panelInput,    Layout.getGBC( 0, 0, 1, 1, 1.0, 1.0
               ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                new Insets( 0, 0, 0, 0), 0, 0));
        this.getContentPane().add(
                panelReportType,Layout.getGBC( 0, 1, 1, 1, 1.0, 1.0
               ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                new Insets( 5, 0, 5, 0), 0, 0));
        this.getContentPane().add(
                panelFields,    Layout.getGBC( 0, 2, 1, 1, 50.0, 50.0
               ,GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                new Insets( 5, 0, 5, 0), 0, 0));
        this.getContentPane().add(
                panelActions, Layout.getGBC( 0, 3, 1, 1, 1.0, 1.0
               ,GridBagConstraints.NORTHEAST, GridBagConstraints.NONE,
                new Insets( 0, 0, 0, 0), 0, 0));
        this.getContentPane().add(
                panelProgress, Layout.getGBC( 0, 4, 1, 1, 1.0, 1.0
               ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                new Insets( 0, 0, 0, 0), 0, 0));
        this.getContentPane().add(
                 panelLog,      Layout.getGBC( 0, 5, 1, 1, 40.0, 40.0
               ,GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                new Insets( 0, 0, 0, 0), 0, 0));

        this.setResizable(true);
        this.pack();

        Dimension size = DcSettings.getValueAsDimension(Repository.settings.stReportingDialogSize);
        setSize(size);

        setCenteredLocation();
    }

    private void openSettingsView() {
        int module = DcSettings.getValueAsInt(Repository.settings.stModule);
        ReportSettings.setCurrentReport(comboReportType.getSelectedIndex(), module);
        SettingsView view = ReportSettings.getView(this);
        view.setVisible(true);
    }

    private class ReportTypeListener implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            storeSettings();
            int module = DcSettings.getValueAsInt(Repository.settings.stModule);
            int reportType = comboReportType.getSelectedIndex();

            buttonSettings.setVisible(reportType != ReportSettings._TEXT_REPORT);

            ReportSettings.setCurrentReport(reportType, module);
            ReportFieldDefinitions definitions =
                    (ReportFieldDefinitions) ReportSettings.getValueAsDefinitionGroup(Repository.reportsettings.stFieldDefinitions);

            fieldTable.setDefinitions(definitions);
            fileField.setValue(ReportSettings.getValueAsString(Repository.reportsettings.stTarget));

            extension = ReportSettings.getValueAsString(Repository.reportsettings.stFileExtension);
            fileField.setFileFilter(new FileFilter(extension));
        }
    }

    private class ResultsButtonAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            showResults();
        }
    }

    private class CloseButtonAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            close();
        }
    }

    private class CancelButtonAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            cancel();
        }
    }

    private class CreateReportAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            createReport();
        }
    }

    private class OpenSettingsViewAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
        	openSettingsView();
        }
    }
}
