/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.core.data;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import javax.swing.JComboBox;

import net.sf.dc.console.MainFrame;
import net.sf.dc.console.views.MasterView;
import net.sf.dc.console.views.View;
import net.sf.dc.core.Repository;
import net.sf.dc.core.modules.DcMediaModule;
import net.sf.dc.core.modules.DcModule;
import net.sf.dc.core.modules.DcModules;
import net.sf.dc.core.modules.IChildModule;
import net.sf.dc.core.modules.contactperson.ContactPersonModule;
import net.sf.dc.core.objects.DcField;
import net.sf.dc.core.objects.DcMediaObject;
import net.sf.dc.core.objects.DcObject;
import net.sf.dc.core.objects.Loan;
import net.sf.dc.core.objects.Picture;
import net.sf.dc.core.objects.contactperson.ContactPerson;
import net.sf.dc.core.objects.template.Templates;
import net.sf.dc.db.DatabaseManager;
import net.sf.dc.messages.Messages;

/**
 * DataManager = Data Crow's persistency layer.
 * Manages what to show in which view. Filters and sorts data.
 *  
 * Objects are loaded on startup and kept in memory. Why?
 * - By default all data is always shown on screen. Filters are applied
 *   on these datasets to limit the presented items.
 * - One object can be be shown in more then one view
 * - Object creation (from raw database data) took a significant amount
 *   of time. To increase loading times keeping all objects in memory is
 *   optimal
 * - DcObject objects do not require much memory. The image presentation
 *   in fancy views is the sole responsible thing for the high memeory usage
 *   
 * Note that it is easy to move away from this strategy. Simply remove the
 * datasets hashmap and load data form database on request 
 * (DatabaseManager.runQueryDirect).
 */ 
public class DataManager {

    private static final Map datasets = new HashMap();
    private static final Map listeners = new HashMap();
    
    public DataManager() {
        initialize();
    }

    public static void bindData(MasterView view, int module, DataFilter df, boolean refresh) {
        view.add(get(module, df));
    }
    
    public static void bindData(View view, int module, DataFilter df, boolean refresh) {
        if (view.getRows() == 0 || refresh)
            view.add(get(module, df));
    }
    
    public static void refresh(DcModule module) {
        Collection c = module.loadData();
        datasets.put(new Integer(module.getIndex()), c);
    }
    
    public static void update(DcObject dco, int module) {
        DcObject o = getObject(dco.getModule().getIndex(), dco.getID());
        
        if (o.getModule() instanceof DcMediaModule || o.getModule() instanceof ContactPersonModule)
            updatePictures(dco);
        
        o.reload();
        
        updateRelated(dco, false);
        updateUiComponents(o.getModule().getIndex());
        updateView(o, 0, module, MainFrame._SEARCHTAB);
    }    
    
    public static void add(DcObject dco, int module) {
    	if (dco.getModule() instanceof DcMediaModule || dco.getModule() instanceof ContactPersonModule)
            updatePictures(dco);
        
        Object values = datasets.get(new Integer(dco.getModule().getIndex()));
        if (values instanceof Collection) {
            Collection dataset = (Collection) values;
            dataset.add(dco);
        } else if (values instanceof Map) {
            Map dataset = (Map) values;
            Object key = getKey(dco);
            
            Collection c = (Collection) dataset.get(key);
            c = c == null ? new ArrayList() : c;
            c.add(dco);
            dataset.put(key, c);
        }

        updateRelated(dco, false);
        updateUiComponents(dco.getModule().getIndex());
        updateView(dco, 1, module, MainFrame._INSERTTAB);
    }  
    
    public static void remove(DcObject dco, int module) {
        Object values = datasets.get(new Integer(dco.getModule().getIndex()));
        if (values instanceof Collection) {
            Collection dataset = (Collection) values;
            dataset.remove(dco);
        } else if (values instanceof Map) {
            Map dataset = (Map) values;
            Object key = getKey(dco);
            Collection c = (Collection) dataset.get(key);
            if (c != null) 
                c.remove(dco);
        }
        
        updateRelated(dco, true);
        updateUiComponents(dco.getModule().getIndex());
        updateView(dco, 2, module, MainFrame._SEARCHTAB);
    }    
    
    private static void updateView(DcObject dco, int mode, int module, int tab) {
        if (!(dco instanceof DcMediaObject  || dco instanceof ContactPerson))
            return;
        
        DcModule m = DcModules.getModule(module);
        if (m.getIndex() != DcModules._GENERIC)
            m = dco.getModule();
    
        if (tab == MainFrame._INSERTTAB) {
        	if (mode == 0) {
        		m.getCurrentInsertView().updateRow(dco.getID(), dco, true, true, false);
        	} else if (mode == 1) {
                
                if (DcModules.getModule(module).getSearchView() != null)
                    DcModules.getModule(module).getSearchView().add(dco);
                
                if (DcModules.getModule(module).getInsertView() != null)
                    DcModules.getModule(module).getInsertView().removeRows(new String[] {dco.getID()});
        	}
        } else if (tab == MainFrame._SEARCHTAB) {
            for (Iterator iter = DcModules.getModules(); iter.hasNext(); ) {
                try {
                    DcModule mod = (DcModule) iter.next();
                    MasterView masterView = mod.getSearchView();
                    if (masterView != null) {
                        switch (mode) {
                        case 0:
                            masterView.updateRow(dco.getID(), dco, true, true, false);
                            break;
                        case 1:
                            masterView.add(dco);
                            break;
                        case 2:
                            masterView.removeRows(new String[] {dco.getID()});
                            break;
                        }
                    }
                } catch (Exception ignore) {}
            }            
        }    
    }
    
    private static Object getKey(DcObject dco) {
        if (dco instanceof Picture)
            return dco.getValue(Picture._A_OBJECTID);
        else if (dco instanceof Loan)
            return dco.getValue(Loan._D_OBJECTID);
        else if (dco.getParentReferenceFieldIndex() > -1)
            return dco.getValue(dco.getParentReferenceFieldIndex());

        return null;
    }    
    
    private static void updateRelated(DcObject dco, boolean delete) {
        if (delete) {
            
            if (dco.getModule() instanceof DcMediaModule || dco.getModule() instanceof ContactPersonModule) {
                Map pictures = (Map) datasets.get(new Integer(DcModules._PICTURE));
                pictures.remove(dco.getID());
            }

            if (dco.getModule().canBeLended()) {
                Map loans = (Map) datasets.get(new Integer(DcModules._LOAN));
                loans.remove(dco.getID());
            }

        } else {
            if (dco instanceof Loan)
                updateLoanedItems((Loan) dco);
        }
    }   

    private static void updatePictures(DcObject dco) {
        Collection pictures = new ArrayList();
        for (Iterator iter = dco.getFields().iterator(); iter.hasNext(); ) {
            DcField field = (DcField) iter.next();
            if (field.getValueType() == Repository.datatypes._PICTURE) {
                Picture picture = (Picture) dco.getValue(field.getIndex());
                if (picture != null && (picture.isUpdated() || picture.isNew() || picture.isDeleted()))
                    pictures.add(picture);
            }
        }

        for (Iterator iter = pictures.iterator(); iter.hasNext(); ) {
            Picture picture = (Picture) iter.next();
            if (picture.isNew()) {
                Map pictureMap = (Map) datasets.get(new Integer(DcModules._PICTURE));
                Collection c = (Collection) pictureMap.get(dco.getID());
                c = c == null ? new ArrayList() : c;
                picture.markAsUnchanged();
                c.add(picture);
                pictureMap.put(dco.getID(), c);
            } else if (picture.isUpdated()) {
                Map pictureMap = (Map) datasets.get(new Integer(DcModules._PICTURE));
                ArrayList c = (ArrayList) pictureMap.get(dco.getID());
                c = c == null ? new ArrayList() : c;
                
                picture.setValue(Picture._D_IMAGE, null);
                if (c.indexOf(picture) > -1) {
                   DcObject o = (DcObject) c.get(c.indexOf(picture));
                   o.copy(picture);
                   o.markAsUnchanged();
                }
            } else if (picture.isDeleted()) {
                Map pictureMap = (Map) datasets.get(new Integer(DcModules._PICTURE));
                ArrayList c = (ArrayList) pictureMap.get(dco.getID());
                c = c == null ? new ArrayList() : c;
                c.remove(picture);
            }
        }
    }
    
    private static void updateLoanedItems(Loan loan) {
        String parentID = (String) loan.getValue(Loan._D_OBJECTID);
        if (parentID != null) {
            DcObject parent = null;
            for (Iterator iter = DcModules.getModules(); iter.hasNext(); ) {
                DcModule module = (DcModule) iter.next();
                
                if (module.canBeLended())
                    parent = getObject(module.getIndex(), parentID, true);
                
                if (parent != null)
                    break;
            }
            
            if (parent != null) {
                parent.reload();
                updateView(parent, 0, parent.getModule().getIndex(), MainFrame._SEARCHTAB);
            }
        }
    }
    
    private static void updateUiComponents(int module) {
    	Collection c = (Collection) listeners.get(new Integer(module));
    	if (c != null) {
    		for (Iterator iter = c.iterator(); iter.hasNext(); ) {
    			JComboBox combo = (JComboBox) iter.next();
    			
                try {
                    Object o = combo.getSelectedItem();
                    combo.removeAllItems();
                    combo.addItem("");

                    DcObject[] objects = get(module, null);
                    for (int i = 0; i < objects.length; i++) {
                        combo.addItem(objects[i]);
                    }

                    if (o != null)
                    	combo.setSelectedItem(o);
                    else
                    	combo.setSelectedIndex(0);

                } catch (Exception exp) {
                    Messages.add(exp);
                    combo.setSelectedIndex(0);
                }
    		}
    	}
    }
    
    public static void registerUiComponent(JComboBox component, int module) {
    	Collection c = (Collection) listeners.get(new Integer(module));
    	c = c == null ? new ArrayList() : c;
    	c.add(component);
    	listeners.put(new Integer(module), c);
    	
    	updateUiComponents(module);
    }
    
    public static int contains(int module, DataFilter df) {
        DcObject[] objects = get(module, df);
        return objects != null ? objects.length : 0;
    }
    
    public static Collection getLoans(String parentID) {
        Map loans = (Map) datasets.get(new Integer(DcModules._LOAN));
        Collection c = (Collection) loans.get(parentID);
        return c != null ? c : new ArrayList();
    }
    
    public static Loan getCurrentLoan(String parentID) {
        Collection loans = getLoans(parentID);
        
        if (loans != null) {
            for (Iterator iter = loans.iterator(); iter.hasNext(); ) {
                Loan loan = (Loan) iter.next();
                if (loan.getValue(Loan._B_ENDDATE) == null)
                    return loan;
            }
        }
        
        return new Loan();
    }
    
    public static DcObject getObjectForDisplayValue(int module, String s) {
        return getObject(module, s, false);
    }    
    
    public static DcObject getObject(int module, String ID) {
        return getObject(module, ID, true);
    }    
    
    private static DcObject getObject(int module, String s, boolean searchOnId) {
        if (s == null || s.length() == 0)
            return null;
        
        Object values = datasets.get(new Integer(module));
        if (values instanceof ArrayList) {
            ArrayList dataset = (ArrayList) values;
            for (Iterator iter = dataset.iterator(); iter.hasNext(); ) {
                DcObject dco = (DcObject) iter.next();
                
                if (searchOnId && dco.getID().equals(s)) 
                    return dco;
                else if (!searchOnId && dco.toString().equals(s))
                    return dco; 
            }
        } else if (values instanceof Map) {
            Map dataset = (Map) values;
            for (Iterator iterSet = dataset.keySet().iterator(); iterSet.hasNext(); ) {
                Object key = iterSet.next();
                ArrayList c = (ArrayList) dataset.get(key);
                for (Iterator iter = c.iterator(); iter.hasNext(); ) {
                    DcObject dco = (DcObject) iter.next();
                    
                    if (searchOnId && dco.getID().equals(s)) 
                        return dco;
                    else if (!searchOnId && dco.toString().equals(s))
                        return dco; 
                }
            }
        }
        return null;
    }
    
    public static Collection getPictures(String parentID) {
        Map dataset = (Map) datasets.get(new Integer(DcModules._PICTURE));
        Collection pictures = (Collection) dataset.get(parentID);
        return pictures == null ? new ArrayList() : pictures; 
    }
    
    public static Collection getChildren(String parentID, int childModule) {
        Map dataset = (Map) datasets.get(new Integer(childModule));
        Collection children = (Collection) dataset.get(parentID);
        
        DcModule module =  DcModules.getModule(childModule);
        DataFilterOptions dfo = new DataFilterOptions(
                new DcField[] {module.getField(((IChildModule) module).getSortIdx())}, false, false);
        DataFilter df = new DataFilter(childModule, module.getDcObject(), dfo);
        children = children == null ? new ArrayList() : children;
        df.sort(children);
        
        return children;
    }
    
    public static DcObject[] get(int module, DataFilter df) {
    	Object values;
    	if (module == DcModules._GENERIC) {
    		Collection c = new ArrayList();
    		
    		for (Iterator iter = DcModules.getModules(); iter.hasNext(); ) {
    			DcModule m = (DcModule) iter.next();
    			if (m.getIndex() != module && m.isTopModule() && m instanceof DcMediaModule) {
    				DcObject[] objects = get(m.getIndex(), df);
    				for (int i = 0; i < objects.length; i++)
    					c.add(objects[i]);
    			}
    		}
    		values = c;
    	} else {
    		values = datasets.get(new Integer(module));
    	}
        
        Collection c;
        if (values instanceof Map) {
            c = new ArrayList();   //;
        	Collection n = ((Map) values).values();
        	for (Iterator iter = n.iterator(); iter.hasNext(); ) {
        		c.addAll((Collection) iter.next()); 
        	}
        } else {
            Collection dataset = (Collection) values;
            dataset = dataset == null ? new ArrayList() : dataset;
            c = new ArrayList();
            
            DcObject[] objects = new DcObject[dataset.size()];
            objects = (DcObject[]) dataset.toArray(objects);
            for (int i = 0; i < objects.length; i++ ) {
                DcObject dco = objects[i];
                if (df == null || (df != null && df.applies(dco)))
                    c.add(dco);
            }
        }
        
        if (df != null) 
            df.sort(c);
        
        DcObject[] objects = new DcObject[c.size()];
        return (DcObject[]) c.toArray(objects);
    }    
    
    
    /**********************************
     * Initialization
     **********************************/
    public void setPictures() {
        Map pics = new HashMap();
        Collection pictures = DcModules.getModule(DcModules._PICTURE).loadData();
        for (Iterator iter = pictures.iterator(); iter.hasNext(); ) {
            Picture pic = (Picture) iter.next();
            Object key = pic.getValue(Picture._A_OBJECTID);
            
            Collection c = (Collection) pics.get(key);
            if (c == null)
                c = new ArrayList();
            
            c.add(pic);
            pics.put(key, c);
         }
         
         add(DcModules._PICTURE, pics);
    }
    
    public void setLoans() {
        Map loanMap = new HashMap();
        Collection loans = DcModules.getModule(DcModules._LOAN).loadData();
        for (Iterator iter = loans.iterator(); iter.hasNext(); ) {
            Loan loan = (Loan) iter.next();
            Object key = loan.getValue(Loan._D_OBJECTID);
            
            Collection c = (Collection) loanMap.get(key);
            if (c == null)
                c = new ArrayList();
            
            c.add(loan);
            loanMap.put(key, c);
         }
         
         add(DcModules._LOAN, loanMap);
    }    
    
    public void setAudioTracks() {
        Map map = new HashMap();

        DcModule module = DcModules.getModule(DcModules._AUDIOTRACK);
        Collection tracks = module.loadData();
        for (Iterator iter = tracks.iterator(); iter.hasNext(); ) {
            DcObject dco = (DcObject) iter.next();
            Object key = dco.getValue(dco.getParentReferenceFieldIndex());
            Collection c = (Collection) map.get(key);
            if (c == null)
                c = new ArrayList();
            
            c.add(dco);
            map.put(key, c);
         }
         
         add(DcModules._AUDIOTRACK, map);
    }
    
    public void setMusicTracks() {
        Map map = new HashMap();

        DcModule module = DcModules.getModule(DcModules._MUSICTRACK);
        Collection tracks = module.loadData();
        for (Iterator iter = tracks.iterator(); iter.hasNext(); ) {
            DcObject dco = (DcObject) iter.next();
            Object key = dco.getValue(dco.getParentReferenceFieldIndex());
            
            Collection c = (Collection) map.get(key);
            if (c == null)
                c = new ArrayList();
            
            c.add(dco);
            map.put(key, c);
         }
         
         add(DcModules._MUSICTRACK, map);
    }    
    
    private void initialize() {
        datasets.clear();
        
        ThreadGroup group = new ThreadGroup("data-fetchers");
        Collection fetchers = new ArrayList();
        
        // Other modules depend on this data : load first
        setPictures();
        setLoans();
        setAudioTracks();
        setMusicTracks();
        
        for (Iterator iter = DcModules.getAllModules(); iter.hasNext();) {
            DcModule module = (DcModule) iter.next();
            
            if (!datasets.containsKey(new Integer(module.getIndex())) && !module.isTopModule()) {
                Thread fetcher = new DataFetcher(module, group);
                fetcher.start();
                fetchers.add(fetcher);
            }
        }
        
        for (Iterator iter = fetchers.iterator(); iter.hasNext();) {
            Thread thread = (Thread) iter.next();
            try {
                thread.join();
            } catch (InterruptedException e) {}
        }         
        
        fetchers.clear();
        for (Iterator iter = DcModules.getAllModules(); iter.hasNext();) {
            DcModule module = (DcModule) iter.next();
            
            if (!datasets.containsKey(new Integer(module.getIndex())) && module.isTopModule()) {
                Thread fetcher = new DataFetcher(module, group);
                fetcher.start();
                fetchers.add(fetcher);
            }
        }        
        
        for (Iterator iter = fetchers.iterator(); iter.hasNext();) {
            Thread thread = (Thread) iter.next();
            try {
                thread.join();
            } catch (InterruptedException e) {}
        }    
        
        Templates.refresh();
    }
    
    private void add(int module, Collection objects) {
        datasets.put(new Integer(module), objects);
    }

    private void add(int module, Map objects) {
        datasets.put(new Integer(module), objects);
    }
    
    final class DataFetcher extends Thread {
    	
        private final DcModule module;
        
        public DataFetcher(DcModule module, ThreadGroup group) {
            super(group, "datafetcher-" + module.getName());
            this.module = module;
        }
        
        public void run() {
        	
        	while (DatabaseManager.getQueueSize() > 0)
        		try { sleep(100); } catch (Exception exp) {}
        	
            add(module.getIndex(), module.loadData());
        }
    }
}
