package net.sf.dc.processes.http;

import java.net.URL;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;

import net.sf.dc.core.objects.DcObject;
import net.sf.dc.core.objects.audiocd.AudioCD;
import net.sf.dc.core.objects.audiocd.AudioTrack;
import net.sf.dc.core.objects.onlinesearch.DcRegion;
import net.sf.dc.core.objects.onlinesearch.DcSearchMode;
import net.sf.dc.http.HttpConnection;
import net.sf.dc.http.covers.AmazonCoverSource;
import net.sf.dc.resources.DcResources;
import net.sf.ext.util.StringUtils;

public class MusicBrainzUpdate extends InternetUpdateProcess {

    private final DcSearchMode mode;
    private final String server;

    private String searchString;

    public MusicBrainzUpdate(IOnlineUpdateListener listener, String searchString, DcSearchMode mode, DcRegion region) {
        super(listener);
        
        this.mode = mode;
        this.searchString = searchString.replace(' ', '+');
        this.server = region.getUrl();
    }

    public void run() {
        Collection parts = getParts();
        
        listener.processingTotal(parts.size());

        if (parts.size() == 0) {
            listener.addWarning(DcResources.getText("msgNoResultsForKeywords", searchString));
            listener.stopped();
            return;
        }

        listener.addMessage(DcResources.getText("msgFoundXResults", String.valueOf(parts.size())));
        listener.addMessage(DcResources.getText("msgStartParsingXResults", String.valueOf(parts.size())));
        
        int counter = 0;
        for (Iterator iter = parts.iterator(); iter.hasNext() && !isCancelled(); counter++) {
            try {
                DcObject dco = parseInformation((String) iter.next());
                listener.addMessage(DcResources.getText("msgParsingSuccessfull", dco.getValue(AudioCD._A_TITLE).toString()));
                listener.addObject(dco);
            } catch (Exception exp) {
                listener.addMessage(DcResources.getText("msgParsingError", exp.getMessage()));
            }
            
            listener.processed(counter + 1);
        }
        listener.stopped();
    }
    
    private Collection getParts() {
        Collection parts = new ArrayList();

        try {
            searchString = searchString.replaceAll(" ", "%20");
            
            listener.addMessage(DcResources.getText("msgConnectingToServer", server));

            String httpUrl = server + "/release/?type=xml&limit=10";
            httpUrl += mode.getSearchCommand(searchString);
            URL url = new URL(httpUrl);

            HttpConnection connection = new HttpConnection(url);
            String page = connection.getString();
            connection.close();

            parts.addAll(getValues(page, "<release id=" + '"', "</release>"));

        } catch (Exception e) {
            listener.addError(DcResources.getText("msgCouldNotConnectToMusicBrainz"));
        }
        
        return parts;
    }
    
    private Collection getValues(String rawResult, String startTag, String endTag) {
        Collection result = new ArrayList();
        
        if (rawResult.indexOf(startTag) > -1) {
            String page = rawResult.substring(rawResult.indexOf(startTag), 
                                                   rawResult.lastIndexOf(endTag) + endTag.length());
            
            int indexBegin = page.indexOf(startTag);
            int indexEnd = 0;
            while(indexBegin > -1) {
                indexBegin = page.indexOf(startTag);;
                indexEnd = page.indexOf(endTag);
    
                String s = page.substring(indexBegin, indexEnd + endTag.length());
                result.add(StringUtils.convertHttpString(s));
                
                indexEnd = indexEnd + endTag.length();
                page = page.substring(indexEnd, page.length());
                indexBegin = page.indexOf(startTag);
            }
        }
        return result;
    }     

    public AudioCD parseInformation(String part) throws Exception {
        AudioCD audioCD = new AudioCD();
        
        audioCD.setValue(AudioCD._A_TITLE, getTitle(part));
        audioCD.setValue(AudioCD._F_ARTIST, getArtists(part));
        
        String mbAlbumId = getId(part);
        
        URL url = new URL(server + "/release/" + mbAlbumId + "?type=xml&inc=tracks");

        HttpConnection connection = new HttpConnection(url);
        String page = connection.getString();
        connection.close();
        
        addTracks(audioCD, page);
        
        AmazonCoverSource coverSearcher = new AmazonCoverSource(listener, audioCD);
        coverSearcher.search();
        
        return audioCD;
    }
    
    private String getTitle(String part) {
        return StringUtils.convertHttpString(StringUtils.getValueBetweenStrings("<title>", "</title>", part));
    }

    private String getId(String part) {
        String key = "<release id=" + '"';
        int idx = part.indexOf(key);
        String s = part.substring(idx + key.length(), part.length());
        return s.substring(0, s.indexOf('"'));
    }
    
    private void addTracks(AudioCD audioCD, String part) {
        Collection c = getValues(part, "<track id", "</track>");
        int number = 1;
        for (Iterator iter = c.iterator(); iter.hasNext() && !isCancelled(); number++) {
            String s = (String) iter.next();
            AudioTrack track = new AudioTrack();
            
            String title = StringUtils.convertHttpString(StringUtils.getValueBetweenStrings("<title>", "</title>", s));
            track.setValue(AudioTrack._A_TITLE, title);
            track.setValue(AudioTrack._F_TRACKNUMBER, new Integer(number));

            try {
                String duration = StringUtils.convertHttpString(StringUtils.getValueBetweenStrings("<duration>", "</duration>", s));
                int seconds = Integer.parseInt(duration.trim()) / 1000;
                track.setValue(AudioTrack._H_PLAYLENGTH, new Integer(seconds));
            } catch (Exception ignore) {}
            
            audioCD.addChild(track);
        }
        
        String key = "life-span begin=" + '"';
        int idx = part.indexOf(key);
        if (idx > -1) {
            String s = part.substring(idx + key.length(), part.length());
            
            boolean success = false;
            try {
                String year = s.substring(0, s.indexOf("-"));
                audioCD.setValue(AudioCD._C_YEAR, new Integer(year));
                success = true;
            } catch (Exception ignore) {}

            if (!success) {
                try {
                    String year = s.substring(0, s.indexOf('"'));
                    audioCD.setValue(AudioCD._C_YEAR, new Integer(year));
                    success = true;
                } catch (Exception ignore) {}
            }
        }
    }
    
    private String getArtists(String part) {
        Collection artists = getValues(part, "<artist", "</artist>");
        String result = "";
        int counter = 0;
        for (Iterator iter = artists.iterator(); iter.hasNext(); counter++) {
            String s = (String) iter.next();
            
            if (counter > 0)
                result += ",";
            
            result += StringUtils.convertHttpString(StringUtils.getValueBetweenStrings("<name>", "</name>", s));
        }
        
        return result;
    }
}
