/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.reporting;

import java.awt.Color;
import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.util.Iterator;
import java.util.Properties;

import net.sf.dc.core.IconLibrary;
import net.sf.dc.core.Repository;
import net.sf.dc.core.modules.DcModule;
import net.sf.dc.core.objects.DcObject;
import net.sf.dc.core.objects.Picture;
import net.sf.dc.settings.ReportSettings;
import net.sf.dc.settings.definitions.ReportFieldDefinition;
import net.sf.dc.settings.definitions.ReportFieldDefinitions;
import net.sf.ext.util.Utilities;

import com.lowagie.text.BadElementException;
import com.lowagie.text.Cell;
import com.lowagie.text.Chunk;
import com.lowagie.text.Document;
import com.lowagie.text.Font;
import com.lowagie.text.Image;
import com.lowagie.text.Table;
import com.lowagie.text.html.HtmlWriter;

public abstract class HtmlReport extends Report {

    private String path = null;
    
    Properties tableProperties = new Properties();
    Properties tabelHeaderProperties = new Properties();
    
    private String getFullyQualifiedFile(String filename) {
        if (path == null) {
            path = target.toString().substring(0, target.toString().lastIndexOf(File.separator) + 1);
            path += "images" + File.separator;
            File file = new File(path);
            file.mkdirs();
        }
        
        return path + filename;
    }
    
    public void initializeWriter(Document document, FileOutputStream out) throws Exception {
        HtmlWriter writer = HtmlWriter.getInstance(document, new BufferedOutputStream(out));
        writer.setImagepath("./images/");
    }    
    
    protected float[] getWidths(Table table, DcObject dco, ReportFieldDefinitions definitions) {
        float[] widths = new float[definitions.getEnabledCount()];
        int i = 0;
        int totalWeight = 0;
        for (Iterator iter = definitions.iterator(); iter.hasNext(); ) {
            ReportFieldDefinition definition = (ReportFieldDefinition) iter.next();
            
            if (definition.isEnabled()) {
                widths[i++] = definition.getWeight();
                totalWeight += definition.getWeight();
            }
        }
        
        for (int j = 0; j < widths.length; j++) {
                widths[j] = widths[j] / totalWeight * 100;
        }
        
        return widths;
    }    

    protected void initialize() throws Exception {
        super.initialize();
        
        
        Color textBorderColor = ReportSettings.getValueAsColor(Repository.reportsettings.stTextBorderColor);
        int textBorderBottom = ReportSettings.getValueAsBoolean(Repository.reportsettings.stTextBorderBottom) ? 1 : 0;        
        int textBorderTop = ReportSettings.getValueAsBoolean(Repository.reportsettings.stTextBorderTop) ? 1 : 0;
        int textBorderLeft = ReportSettings.getValueAsBoolean(Repository.reportsettings.stTextBorderLeft) ? 1 : 0;
        int textBorderRight = ReportSettings.getValueAsBoolean(Repository.reportsettings.stTextBorderRight) ? 1 : 0;

        
        
        String borderColor = Utilities.getHexColor(textBorderColor);
        tableProperties.put("style", "border-bottom-style:solid;border-color:"+ borderColor +
                            ";border-bottom-width:" + textBorderBottom +
                            ";border-left-style:solid;border-left-width:" + textBorderLeft +
                            ";border-right-style:solid;border-right-width:" + textBorderRight +
                            ";border-top-style:solid;border-top-width:" + textBorderTop);
        
        int labelBorderBottom = ReportSettings.getValueAsBoolean(Repository.reportsettings.stLabelBorderBottom) ? 1 : 0;        
        int labelBorderTop = ReportSettings.getValueAsBoolean(Repository.reportsettings.stLabelBorderTop) ? 1 : 0;
        int labelBorderLeft = ReportSettings.getValueAsBoolean(Repository.reportsettings.stLabelBorderLeft) ? 1 : 0;
        int labelBorderRight = ReportSettings.getValueAsBoolean(Repository.reportsettings.stLabelBorderRight) ? 1 : 0;
        Color labelBorderColor = ReportSettings.getValueAsColor(Repository.reportsettings.stLabelBorderColor);
        
        String borderHeaderColor = Utilities.getHexColor(labelBorderColor);
        tabelHeaderProperties.put("style", "border-bottom-style:solid;border-color:"+ borderHeaderColor +
                                  ";border-bottom-width:" + labelBorderBottom +
                                  ";border-left-style:solid;border-left-width:" + labelBorderLeft +
                                  ";border-right-style:solid;border-right-width:" + labelBorderRight +
                                  ";border-top-style:solid;border-top-width:" + labelBorderTop);
        
    }
    
    protected Cell getLabelCell(String label, ReportFieldDefinition definition) throws BadElementException {
        Chunk chunk = new Chunk(label);
        Cell cell = new Cell(chunk);
        cell.setVerticalAlignment(Cell.ALIGN_TOP);
        cell.setMarkupAttributes(tabelHeaderProperties);
        
        Color labelColor = ReportSettings.getValueAsColor(Repository.reportsettings.stLabelColor);
        Color labelColorBg = ReportSettings.getValueAsColor(Repository.reportsettings.stLabelBackgroundColor);
        Font labelFont = getFont(ReportSettings.getValueAsFont(Repository.reportsettings.stLabelFont), labelColor);
        
        chunk.setFont(labelFont);
        
        cell.setHorizontalAlignment(getAlignment(definition));
        cell.setBackgroundColor(labelColorBg);

        return cell;
    }    
    
    protected void setTextCellDefaults(Cell cell, ReportFieldDefinition definition) {
        cell.setHorizontalAlignment(getAlignment(definition));
        cell.setVerticalAlignment(Cell.ALIGN_TOP);
        
        Color textColorBg = ReportSettings.getValueAsColor(Repository.reportsettings.stTextBackgroundcolor);
        
        cell.setBackgroundColor(textColorBg);
        cell.setMarkupAttributes(tableProperties);       
        cell.setBorderWidth(0);
        cell.setBorder(0);
    }    
    
    protected Cell getRatingCell(Object value, ReportFieldDefinition definition) throws Exception {
        Cell cell = new Cell();

        if (value != null) {
            int rating =  ((Integer) value).intValue();
            String filename = IconLibrary.picPath + "rating" + rating + ".gif";
            byte[] bytes = Utilities.readFile(new File(filename));
        
            String file = "rating" + rating + ".gif";
            Image image = getImage(bytes, getFullyQualifiedFile(file), file);
            cell = new Cell(image);
        }
        
        setTextCellDefaults(cell, definition);
        return cell;
    }

    protected Cell getModuleCell(DcModule module, ReportFieldDefinition definition) throws Exception {
        Cell cell = new Cell(module.getName());
        setTextCellDefaults(cell, definition);
        return cell;
    }    
    
    protected Cell getAvailabilityCell(Object value, ReportFieldDefinition definition) throws Exception {
        Cell cell = new Cell();

        if (value != null) {
            Boolean b = new Boolean(value.toString());
            
            String file = b.booleanValue() ? "checked.gif" : "unchecked.gif";
            byte[] bytes = Utilities.readFile(new File(IconLibrary.picPath + file));
            Image image = getImage(bytes, getFullyQualifiedFile(file), file);
            cell = new Cell(image);

            setTextCellDefaults(cell, definition);            
        }
        return cell;
    }     
    
    protected Image getImage(byte[] bytes, String fullPath, String filename) throws Exception {
        Image image = null;
        
        boolean exists = new File(fullPath).exists();
        if (bytes != null && bytes.length > 10 && !exists) 
            Utilities.writeImageToFile(bytes, fullPath);

        image = Image.getInstance(fullPath);
        
        return image;
    }
    
    
    protected Image getImage(DcObject dco, int field) throws Exception {
        Image image = null;
        try {
            String file = dco.getID() + "_" + dco.getField(field).getDatabaseFieldName() + ".jpg";
            String fullPath = getFullyQualifiedFile(file);
            
            Picture picture = (Picture) dco.getValue(field);
            picture.loadImage();
            if (picture != null) {
                image = getImage((byte[]) picture.getValue(Picture._D_IMAGE), fullPath, file);
                image.scaleToFit(200, 150);
            }
        } catch (Exception exp) {}
        return image;
    }     
    
    protected void addFrontPage(Document document) throws Exception {}     
}

