/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.reporting;

import java.awt.Color;
import java.io.BufferedOutputStream;
import java.io.FileOutputStream;
import java.util.Iterator;

import javax.swing.ImageIcon;

import net.sf.dc.core.IconLibrary;
import net.sf.dc.core.Repository;
import net.sf.dc.core.modules.DcModule;
import net.sf.dc.core.objects.DcObject;
import net.sf.dc.messages.Messages;
import net.sf.dc.settings.ReportSettings;
import net.sf.dc.settings.definitions.ReportFieldDefinition;
import net.sf.dc.settings.definitions.ReportFieldDefinitions;
import net.sf.ext.util.Utilities;

import com.lowagie.text.BadElementException;
import com.lowagie.text.Cell;
import com.lowagie.text.Chunk;
import com.lowagie.text.Document;
import com.lowagie.text.Font;
import com.lowagie.text.FontFactory;
import com.lowagie.text.Image;
import com.lowagie.text.Paragraph;
import com.lowagie.text.Table;
import com.lowagie.text.pdf.PdfWriter;

public abstract class PdfReport extends Report {
    
    public void initializeWriter(Document document, FileOutputStream out) throws Exception {
        PdfWriter.getInstance(document, new BufferedOutputStream(out));
    }    
    
    protected Cell getRatingCell(Object value, ReportFieldDefinition definition) throws Exception {
        ImageIcon icon = Utilities.getRatingIcon(value);
        
        Cell cell;
        if (icon != null) {
            Image image = Image.getInstance(icon.getImage(), null, false);
            cell = new Cell(image);
        } else {
            cell = new Cell();
        }
        
        setTextCellDefaults(cell, definition);
        return cell;
    }

    protected Cell getModuleCell(DcModule module, ReportFieldDefinition definition) throws Exception {
        ImageIcon icon = module.getIconSmall();
        Image image = Image.getInstance(icon.getImage(), null, false);
        Cell cell = new Cell();
        Chunk chunk1 = new Chunk(image, 0, 0);
        Chunk chunk2 = new Chunk(" " + module.getName());
        cell.addElement(chunk1);
        cell.addElement(chunk2);
        
        setTextCellDefaults(cell, definition);
        
        return cell;
    }    
    
    protected Cell getAvailabilityCell(Object value, ReportFieldDefinition definition) throws Exception {
        Cell cell = new Cell();
        if (value != null) {
            Boolean b = new Boolean(value.toString());
            ImageIcon icon = b.booleanValue() ? IconLibrary._icoChecked : IconLibrary._icoUnchecked;
            Image image = Image.getInstance(icon.getImage(), null, false);
            cell = new Cell(image);
            setTextCellDefaults(cell, definition);
        }
        return cell;
    }     
    
    protected Cell getImageCell(ImageIcon icon, ReportFieldDefinition definition, int width, int height, boolean inRow) throws Exception {
        
        Cell cell;
        
        if (icon != null) {
            Image image = Image.getInstance(icon.getImage(), null, false);
            if (icon.getIconWidth() > width || icon.getIconHeight() > height)
                image.scaleToFit(width, height);
        
            cell = new Cell(image);
        } else {
            cell = new Cell();
        }

        cell.setBorder(0);
        
        if (inRow) setTextCellDefaults(cell, definition);
        
        return cell;
    } 
    
    
     protected Cell getLabelCell(String label, ReportFieldDefinition definition) throws BadElementException {
        int labelBorderBottom = ReportSettings.getValueAsBoolean(Repository.reportsettings.stLabelBorderBottom) ? 1 : 0;        
        int labelBorderTop = ReportSettings.getValueAsBoolean(Repository.reportsettings.stLabelBorderTop) ? 1 : 0;
        int labelBorderLeft = ReportSettings.getValueAsBoolean(Repository.reportsettings.stLabelBorderLeft) ? 1 : 0;
        int labelBorderRight = ReportSettings.getValueAsBoolean(Repository.reportsettings.stLabelBorderRight) ? 1 : 0;
        Color labelColor = ReportSettings.getValueAsColor(Repository.reportsettings.stLabelColor);
        Color labelColorBg = ReportSettings.getValueAsColor(Repository.reportsettings.stLabelBackgroundColor);
        Font labelFont = getFont(ReportSettings.getValueAsFont(Repository.reportsettings.stLabelFont), labelColor);
        Color labelBorderColor = ReportSettings.getValueAsColor(Repository.reportsettings.stLabelBorderColor);
         
        Chunk chunk = new Chunk(label);
        Cell cell = new Cell(chunk);
        chunk.setFont(labelFont);

        cell.setHorizontalAlignment(getAlignment(definition));
        
        cell.setBackgroundColor(labelColorBg);
        
        cell.setBorderWidthBottom(labelBorderBottom);
        cell.setBorderWidthLeft(labelBorderLeft);
        cell.setBorderWidthRight(labelBorderRight);
        cell.setBorderWidthTop(labelBorderTop);

        cell.setBorderColorBottom(labelBorderColor);
        cell.setBorderColorLeft(labelBorderColor);
        cell.setBorderColorRight(labelBorderColor);
        cell.setBorderColorTop(labelBorderColor);
        return cell;
    }

    protected void setWidths(Table table, DcObject dco, ReportFieldDefinitions definitions) {
        int[] widths = new int[definitions.getEnabledCount()];
        int i = 0;
        for (Iterator iter = definitions.iterator(); iter.hasNext(); ) {
            ReportFieldDefinition definition = (ReportFieldDefinition) iter.next();
            
            if (definition.isEnabled())
                widths[i++] = definition.getWeight();
        }
        
        try {
            table.setWidth(100f);
            table.setWidths(widths);
        } catch (Exception exp) {
            Messages.add(exp);
        }
    }
    
    protected void setTextCellDefaults(Cell cell, ReportFieldDefinition definition) {
        Color textColorBg = ReportSettings.getValueAsColor(Repository.reportsettings.stTextBackgroundcolor);
        Color textBorderColor = ReportSettings.getValueAsColor(Repository.reportsettings.stTextBorderColor);
        int textBorderBottom = ReportSettings.getValueAsBoolean(Repository.reportsettings.stTextBorderBottom) ? 1 : 0;        
        int textBorderTop = ReportSettings.getValueAsBoolean(Repository.reportsettings.stTextBorderTop) ? 1 : 0;
        int textBorderLeft = ReportSettings.getValueAsBoolean(Repository.reportsettings.stTextBorderLeft) ? 1 : 0;
        int textBorderRight = ReportSettings.getValueAsBoolean(Repository.reportsettings.stTextBorderRight) ? 1 : 0;
        
        cell.setHorizontalAlignment(getAlignment(definition));
        
        cell.setBackgroundColor(textColorBg);
        
        cell.setBorderWidthBottom(textBorderBottom);
        cell.setBorderWidthLeft(textBorderLeft);
        cell.setBorderWidthRight(textBorderRight);
        cell.setBorderWidthTop(textBorderTop);
        
        cell.setBorderColorBottom(textBorderColor);
        cell.setBorderColorLeft(textBorderColor);
        cell.setBorderColorRight(textBorderColor);
        cell.setBorderColorTop(textBorderColor);
    }     
    
    protected void addFrontPage(Document document) throws Exception {
        Paragraph paragraph = new Paragraph();
        paragraph.setAlignment(Paragraph.ALIGN_RIGHT);
        Chunk chunk = new Chunk("a data crow report");
        Font font = FontFactory.getFont("Arial", 8, Font.ITALIC, Color.BLACK);
        chunk.setFont(font);
        paragraph.add(chunk);
        document.add(paragraph);
    
        Paragraph imgParagraph = new Paragraph();
        imgParagraph.setAlignment(Paragraph.ALIGN_CENTER);
    
        Image image = Image.getInstance(IconLibrary._icoReportTitlePage.getImage(), null, false);
        
        Chunk imageChunk = new Chunk(image, -20f, -500f);
        imageChunk.setHorizontalScaling(0f);
        imgParagraph.add(imageChunk);
        document.add(imgParagraph);
    }       
}
