/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.reporting;

import java.awt.Color;
import java.io.File;
import java.io.FileOutputStream;
import java.util.Collection;
import java.util.Iterator;

import net.sf.dc.console.reporting.ReportingDialog;
import net.sf.dc.core.Repository;
import net.sf.dc.core.modules.DcModule;
import net.sf.dc.core.modules.DcModules;
import net.sf.dc.core.objects.DcObject;
import net.sf.dc.messages.Messages;
import net.sf.dc.resources.DcResources;
import net.sf.dc.settings.DcSettings;
import net.sf.dc.settings.ReportSettings;
import net.sf.dc.settings.definitions.ReportFieldDefinition;
import net.sf.dc.settings.definitions.ReportFieldDefinitions;

import com.lowagie.text.BadElementException;
import com.lowagie.text.Cell;
import com.lowagie.text.Chunk;
import com.lowagie.text.Document;
import com.lowagie.text.Font;
import com.lowagie.text.FontFactory;
import com.lowagie.text.HeaderFooter;
import com.lowagie.text.PageSize;
import com.lowagie.text.Phrase;
import com.lowagie.text.Table;

public abstract class Report extends Thread {

    protected ReportFieldDefinitions definitions;
    protected Collection data;
    protected File target;
    protected ReportingDialog dialog;

    protected Document document;
    protected boolean keepOnRunning = true;

    public void run() {
        try {
            create();
        } catch (Exception exp) {
            Messages.add(exp);
            document.close();
            dialog.addMessage(DcResources.getText("msgErrorWhileCreatingReport", exp.toString()));
        } finally {
            data = null;
            dialog.allowActions(true);
        }
    }

    public void cancel() {
        keepOnRunning = false;
    }

    public void compile() {
        super.start();
    }

    protected abstract void setTextCellDefaults(Cell cell, ReportFieldDefinition definition);
    public abstract void create() throws Exception;
    public abstract void initializeWriter(Document document, FileOutputStream out) throws Exception;

    protected void initialize() throws Exception {
        boolean landscape = false;

        try {
            landscape = ReportSettings.getValueAsBoolean(Repository.reportsettings.stPageLayoutLandscape);
        } catch (Exception ignore) {}

        document = landscape ? new Document(PageSize.LETTER.rotate()) : new Document();

        document.addCreator("Data Crow");
        document.addAuthor("Robert Jan van der Waals");
        FileOutputStream out = new FileOutputStream(target);

        initializeWriter(document, out);

        document.open();

        try {
            if (ReportSettings.getValueAsBoolean(Repository.reportsettings.stShowFooter))
                document.setFooter(getDocumentFooter());

            if (ReportSettings.getValueAsBoolean(Repository.reportsettings.stShowHeader))
                document.setHeader(getDocumentHeader());
        } catch (Exception ignore) {}

        addFrontPage(document);
        document.newPage();

        dialog.initProgressBar(data.size());
        dialog.addMessage(DcResources.getText("msgCreatingReportForXObjects", String.valueOf(data.size())));
    }

    protected Font getFont(java.awt.Font f, Color color) {
        try {
            return FontFactory.getFont(f.getName(), f.getSize(), f.getStyle(), color);
        } catch (Exception exp) {
            return null;
        }
    }

    protected Table getTable(int columnCount) throws BadElementException {
        Table datatable = new Table(columnCount);
        datatable.setSpaceInsideCell(1f);
        datatable.setBorder(0);
        datatable.setWidth(100f);
        return datatable;
    }

    protected int getAlignment(ReportFieldDefinition definition) {
        int alignment = Cell.ALIGN_LEFT;

        if (definition != null) {
            if (definition.getAlignment() == ReportFieldDefinition._CENTER) {
                alignment = Cell.ALIGN_CENTER;
            } else if (definition.getAlignment() == ReportFieldDefinition._LEFT) {
                alignment = Cell.ALIGN_LEFT;
            } else {
                alignment = Cell.ALIGN_RIGHT;
            }
        }

        return alignment;
    }

    protected Cell getTextCell(String text, ReportFieldDefinition definition) throws BadElementException {
        Color textColor = ReportSettings.getValueAsColor(Repository.reportsettings.stTextColor);
        Font textFont = getFont(ReportSettings.getValueAsFont(Repository.reportsettings.stTextFont), textColor);

        Chunk chunk = new Chunk(text);
        Cell cell = new Cell(chunk);
        chunk.setFont(textFont);
        setTextCellDefaults(cell, definition);
        cell.setShowTruncation("....");
        cell.setMaxLines(15);
        return cell;
    }

    private HeaderFooter getDocumentFooter() {
        Color fontColor = ReportSettings.getValueAsColor(Repository.reportsettings.stFooterColor);
        Font font = getFont(ReportSettings.getValueAsFont(Repository.reportsettings.stFooterFont), fontColor);
        String value = ReportSettings.getValueAsString(Repository.reportsettings.stFooterValue);

        Chunk chunk1 = new Chunk(value + " ");
        chunk1.setFont(font);
        Chunk chunk2 = new Chunk("");
        chunk2.setFont(font);

        HeaderFooter footer = new HeaderFooter(new Phrase(chunk1),
                                               new Phrase(chunk2));
        footer.setBorder(0);
        Color background = ReportSettings.getValueAsColor(Repository.reportsettings.stFooterBackgroundColor);
        footer.setBackgroundColor(background);
        return footer;
    }

    private HeaderFooter getDocumentHeader() {
        String text = ReportSettings.getValueAsString(Repository.reportsettings.stHeaderValue);
        Chunk chunk = new Chunk(text);

        Color fontColor = ReportSettings.getValueAsColor(Repository.reportsettings.stHeaderColor);
        Font font = getFont(ReportSettings.getValueAsFont(Repository.reportsettings.stHeaderFont), fontColor);
        chunk.setFont(font);
        HeaderFooter header = new HeaderFooter(new Phrase(chunk), false);
        header.setBorder(0);

        Color background = ReportSettings.getValueAsColor(Repository.reportsettings.stHeaderBackgroundColor);
        header.setBackgroundColor(background);
        return header;
    }
    
    protected abstract Cell getLabelCell(String label, ReportFieldDefinition definition) throws BadElementException;
    
    protected Table getChildTable(DcObject parent) throws Exception {
        int module = DcSettings.getValueAsInt(Repository.settings.stModule);

        DcModule childModule = DcModules.getModule(module).getChildModule();

        Table childrenTable = null;
        if (childModule != null) {
            parent.loadChildren();
            Collection children = parent.getChildren();

            if (children != null && children.size() > 0) {
                DcObject child = parent.getModule().getChildModule().getDcObject();
                
                ReportFieldDefinitions defs = parent.getModule().getChildModule().getReportFieldDefinitions();
                childrenTable = getTable(defs.getEnabledCount());
                
                float[] widths = new float[defs.getEnabledCount()];
                int counter = 0;
                for (Iterator iter = defs.iterator(); iter.hasNext(); ) {
                    ReportFieldDefinition def = (ReportFieldDefinition) iter.next();
                    
                    if (def.isEnabled()) {
                        String label = child.getField(def.getIndex()).getLabel();
                        childrenTable.addCell(getLabelCell(label, null));
                        widths[counter++] = def.getWeight();
                    }
                }

                childrenTable.setWidths(widths);

                for (Iterator iter = children.iterator(); iter.hasNext(); ) {
                    DcObject o = (DcObject)  iter.next();
                    
                    for (Iterator iterDefs = defs.iterator(); iterDefs.hasNext(); ) {
                        ReportFieldDefinition def = (ReportFieldDefinition) iterDefs.next();
                        
                        if (def.isEnabled()) {
                            String s = o.getDisplayString(def.getIndex());
                            childrenTable.addCell(getTextCell(s, null));
                        }
                    }
                }
            }
        }
        
        return childrenTable;
    }    

    protected abstract void addFrontPage(Document document) throws Exception;
}
