package net.sf.ext.mediamanager;

/**
 * @(#)FilePropertiesMovie.java 1.0 06.06.05 (dd.mm.yy)
 *
 * Copyright (2003) Mediterranean
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2, or any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with 
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Boston, MA 02111.
 * 
 * Contact: mediterranean@users.sourceforge.net
 **/

import java.io.File;
import java.io.RandomAccessFile;
import java.util.ArrayList;
import java.util.Arrays;

import net.sf.dc.messages.Messages;
import net.sf.ext.util.Utilities;

public class FilePropertiesMovie {

	private String _subtitles = "";
	private int _fileSize = -1;

    // The resolution (width x heigth).
	private String _videoResolution = "";
	private String _videoCodec = "";
	private int _videoRate = 0;
	private int _videoBitRate = 0;
	private int _duration = -1;
	private String _videoName = "";
	private String _audioCodec = "";
    // The audio rate (Hz).
	private int _audioRate = 0;
	// The audio bit rate (kbps).
	private int _audioBitRate = 0;
	private int _audioChannels = 0;
	private String _location = "";
	private String _container = "";
	private ArrayList metaData;

	/**
	 * The Constructor. Gets inicializes all info available (reads contents from file).
	 *
	 * @param filePath A path for the file.
	 **/
	public FilePropertiesMovie(String filePath) throws Exception {
		boolean supported = false;
		FileProperties fileProperties = null;

		try {
			/* The known magic header bytes... */
			final int[][] MAGIC_BYTES = { 
                    { 0x00, 0x00, 0x01, 0xb3 }, // MPEG (video)
					{ 0x00, 0x00, 0x01, 0xba }, // MPEG (video)
					{ 0x52, 0x49, 0x46, 0x46 }, // RIFF (WAV / audio, AVI / video)
					{ 0x4f, 0x67, 0x67, 0x53 }, // OGM 
					{ 0x44, 0x56, 0x44, 0x56 }, // IFO (DVDV)
					{ 0x1a, 0x45, 0xdf, 0xa3 }, // MKV
					{ 0x30, 0x26, 0xb2, 0x75 }  // ASF (audio / video)
			};
			/**
			 * The respective objects.
			 **/
			final FileProperties[] FORMATS = { new FilePropertiesMPEG(),
					new FilePropertiesMPEG(), new FilePropertiesRIFF(),
					new FilePropertiesOGM(), new FilePropertiesIFO(),
					new FilePropertiesMKV(), new FilePropertiesASF() };

			_fileSize = Math.round((new File(filePath).length()) / 1024F / 1024F);
			RandomAccessFile dataStream = new RandomAccessFile(filePath, "r");

			// Gets the header for filetype identification... 
			int[] header = new int[4];
			for (int i = 0; i < header.length; i++) {
				header[i] = dataStream.readUnsignedByte();
            }

			// Finds the right object...
			int format = 0;
			for (format = 0; format < FORMATS.length; format++) {
				if (Arrays.equals(header, MAGIC_BYTES[format])) {
					break;
				}
			}

			// If the right object was found...
			if (format < FORMATS.length) {
				supported = true;

				// Gets the right object...
				fileProperties = FORMATS[format];

				// Sends a process() order..
				fileProperties.process(dataStream);

				File temp = new File(filePath);
				String name = temp.getName().substring(0,temp.getName().lastIndexOf("."));
				name = name.replaceAll(".", " ");
				name = name.replaceAll("_", " ");
				name = name.replaceAll("-", " ");

				setVideoName(name);

				// Gets the processed info...
				_subtitles = fileProperties.getSubtitles();
				_videoResolution = fileProperties.getVideoResolution();
				_videoCodec = fileProperties.getVideoCodec();
				_videoRate = fileProperties.getVideoRate();
				_videoBitRate = fileProperties.getVideoBitRate();
				_duration = fileProperties.getDuration();
				_audioCodec = fileProperties.getAudioCodec();
				_audioRate = fileProperties.getAudioRate();
				_audioBitRate = fileProperties.getAudioBitRate();
				_audioChannels = fileProperties.getAudioChannels();
				_location = filePath;
				_container = fileProperties.getContainer();
				metaData = fileProperties.getMetaData();
			}

            _subtitles = Utilities.fillSubtitles(filePath, _subtitles);
            
			// Closes the input stream... 
			dataStream.close();

		} catch (Exception e) {
			Messages.add(e);

			// The file is corrupted, tries to save the info that may have been found
			if (fileProperties != null) {
				_subtitles = fileProperties.getSubtitles();
				_videoResolution = fileProperties.getVideoResolution();
				_videoCodec = fileProperties.getVideoCodec();
				_videoRate = fileProperties.getVideoRate();
				_videoBitRate = fileProperties.getVideoBitRate();
				_duration = fileProperties.getDuration();
				_audioCodec = fileProperties.getAudioCodec();
				_audioRate = fileProperties.getAudioRate();
				_audioBitRate = fileProperties.getAudioBitRate();
				_audioChannels = fileProperties.getAudioChannels();
			}

			throw new Exception("File is corrupted. Some info may have been saved.");

		} finally {
			if (!supported) {
				throw new Exception("File format not supported.");
			}
		}
	}
    
	/**
	 * The Constructor.
	 **/
	public FilePropertiesMovie(int fileSize, String videoResolution,
			String videoCodec, int videoRate, int duration, String audioCodec,
			int audioRate, int audioChannels) {
		_fileSize = fileSize;
		_videoResolution = videoResolution;
		_videoCodec = videoCodec;
		_videoRate = videoRate;
		_duration = duration;
		_audioCodec = audioCodec;
		_audioRate = audioRate;
		_audioChannels = audioChannels;
	}

	/**
	 * Returns all the components in a string.
	 **/
	public String toString() {
		return "MovieProperties [ fileSize:" + _fileSize + ", "
				+ "vResolution:" + _videoResolution + ", " + "vCodec:"
				+ _videoCodec + ", " + "vRate:" + _videoRate + ", "
				+ "vBitRate:" + _videoBitRate + ", " + "duration:(" + _duration
				+ ") ," + "aCoded:" + _audioCodec + ", " + "aRate:"
				+ _audioRate + ", " + "aBitRate:" + _audioBitRate + ", "
				+ "aChannels:" + _audioChannels + " ]";
	}
    
	public int getVideoWidth() {
		try {
			String width = getVideoResolution().substring(
                           0, getVideoResolution().indexOf("x"));
			return new Integer(width).intValue();
		} catch (Exception exp) {
			return 0;
		}
	}

	public int getVideoHeight() {
		try {
			String height = getVideoResolution().substring(
					getVideoResolution().indexOf("x") + 1,
					getVideoResolution().length());
			return new Integer(height).intValue();
		} catch (Exception exp) {
			return 0;
		}
	}

	/*
	 * Returns the the tag info if it exists
	 */
	public String getMetaDataTagInfo(String tag) {
		String temp = "";
		if (metaData != null) {
			for (int i = 0; i < metaData.size(); i++) {
				temp = ((String) metaData.get(i));
				if (temp.startsWith(tag)) {
					return temp.substring(tag.length() + 1, temp.length());
				}
			}
		}
		return "";
	}

	/**
	 * Returns the subtitles.
	 **/
	public String getSubtitles() {
		return _subtitles;
	}

	/**
	 * Returns the filesize.
	 **/
	public int getFileSize() {
		return _fileSize;
	}

	/**
	 * Returns the resolution.
	 **/
	public String getVideoResolution() {
		return _videoResolution;
	}

	/**
	 * Returns the video codec (vids handler).
	 **/
	public String getVideoCodec() {
		return _videoCodec;
	}

	/**
	 * Returns the video rate.
	 **/
	public int getVideoRate() {
		return _videoRate;
	}

	/**
	 * Returns the video bit rate.
	 **/
	public int getVideoBitRate() {
		return _videoBitRate;
	}

	/**
	 * Returns the duration.
	 **/
	public int getDuration() {
		return _duration;
	}

	/**
	 * Returns the audio codec (auds handler).
	 **/
	public String getAudioCodec() {
		return _audioCodec;
	}

	/**
	 * Returns the audio rate.
	 **/
	public int getAudioRate() {
		return _audioRate;
	}

	/**
	 * Returns the audio bit rate.
	 **/
	public int getAudioBitRate() {
		return _audioBitRate;
	}

	/**
	 * Returns the audio channels.
	 **/
	public int getAudioChannels() {
		return _audioChannels;
	}

	/**
	 * Sets the subtitles.
	 **/
	public void setSubtitles(String subtitles) {
		_subtitles = subtitles;
	}

	/**
	 * Sets the length.
	 **/
	public void setFileSize(int fileSize) {
		_fileSize = fileSize;
	}

	/**
	 * Sets the length.
	 **/
	public void setVideoName(String name) {
		_videoName = name;
	}

	/**
	 * Sets the resolution.
	 **/
	public void setVideoResolution(String videoResolution) {
		_videoResolution = videoResolution;
	}

	/**
	 * Sets the video codec (vids handler).
	 **/
	public void setVideoCodec(String videoCodec) {
		_videoCodec = videoCodec;
	}

	/**
	 * Sets the video rate.
	 **/
	public void setVideoRate(int videoRate) {
		_videoRate = videoRate;
	}

	/**
	 *  Sets the video bit rate.
	 **/
	public void setVideoBitRate(int videoBitRate) {
		_videoBitRate = videoBitRate;
	}

	/**
	 * Sets the duration.
	 **/
	public void setDuration(int duration) {
		_duration = duration;
	}

	/**
	 * Sets the audio codec (auds handler).
	 **/
	public void setAudioCodec(String audioCodec) {
		_audioCodec = audioCodec;
	}

	/**
	 * Sets the audio rate.
	 **/
	public void setAudioRate(int audioRate) {
		_audioRate = audioRate;
	}

	/**
	 * Sets the audio channels.
	 **/
	public void setAudioChannels(int audioChannels) {
		_audioChannels = audioChannels;
	}

	/**
	 * Sets the file location.
	 **/
	public void setFileLocatinon(String location) {
		_location = location;
	}

	/**
	 * Sets the file container.
	 **/
	public String getLocation() {
		return _location;
	}

	/**
	 * Sets the file container.
	 **/
	public void setContainer(String container) {
		_container = container;
	}

	/**
	 * Returns the file container.
	 **/
	public String getContainer() {
		return _container;
	}

	public String getVideoName() {
		return _videoName;
	}
}
