/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.ext.util;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.GraphicsConfiguration;
import java.awt.GraphicsDevice;
import java.awt.GraphicsEnvironment;
import java.awt.HeadlessException;
import java.awt.Image;
import java.awt.Point;
import java.awt.Toolkit;
import java.awt.Transparency;
import java.awt.image.BufferedImage;
import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.RandomAccessFile;
import java.net.URL;
import java.util.Calendar;
import java.util.Random;

import javax.imageio.ImageIO;
import javax.swing.ImageIcon;

import net.sf.dc.core.IconLibrary;
import net.sf.dc.messages.Messages;
import net.sf.dc.resources.DcResources;

public class Utilities {

    private static Random random = new Random();
    
    /**
     * Returns a centered location for a window / form / dialog 
     * @param windowSize size of the window
     * @return centered location
     */
    public static Point getCenteredWindowLocation(Dimension windowSize) {
        Dimension dim = Toolkit.getDefaultToolkit().getScreenSize();
        dim.height = (dim.height-windowSize.height)/2;
        dim.width = (dim.width-windowSize.width) /2;
        return new Point(dim.width, dim.height);
    }
    
    public static boolean sameImage(byte[] img1, byte[] img2) {
        boolean same = img1.length == img2.length;
        if (same) {
            for (int i = 0; i < img1.length; i++) {
                same = img1[i] == img2[i];
                if (!same)
                    break;
            }
        }
        return same;
    }    
    
    public static String fillSubtitles(String filePath, String subtitles) {
        try {
            File f = new File(filePath);
            File dir = f.getParentFile();
            String movieName = f.getName().substring(0,f.getName().lastIndexOf("."));
            String [] allFiles = dir.list();
            for (int i = 0; i < allFiles.length; i++){
                String name = allFiles[i];
                if (    name.startsWith(movieName) && 
                        ( name.toUpperCase().endsWith("SRT") ||
                          name.toUpperCase().endsWith("SUB") ||
                          name.toUpperCase().endsWith("SSA"))) {
                    
                    if (!subtitles.equals("")) {
                        subtitles =subtitles + " ";
                    }
                    subtitles = subtitles + name.substring(movieName.length());
                }
            }
        } catch (Exception exp) {
            Messages.add(exp);
        }
        
        return subtitles;
    }
    
    private static final Calendar calendar = Calendar.getInstance();

    /**
     * Creates a unique ID. Can be used for custom IDs in the database.
     * Based on date / time + random number
     * @return unique ID as String
     */
    public static String getUniqueID() {
        long temp = random.nextLong();
        if (temp < 0l) {
            temp = temp * -1;
        }
        
        StringBuffer sb = new StringBuffer();
        sb.append(calendar.get(Calendar.MONTH) + 1);
        sb.append(calendar.get(Calendar.DAY_OF_MONTH));
        sb.append(temp);
        
        return sb.toString();
    }
    
    /**
     * Retrieved the file extension of a file
     * @param f file to get the extension from
     * @return extenstion or empty string
     */
    public static String getExtension(File f) {
        String name = f.getName().toLowerCase();
        int i = name.lastIndexOf( "." );
        if (i == -1) {
            return "";
        }
        return name.substring( i + 1 );
    }    
    
    public static int getIntegerValue(String s) {
        char[] characters = s.toCharArray();
        String test = "";
        for (int i = 0; i < characters.length; i++) {
            if (Character.isDigit(characters[i])) test += "" + characters[i];
        }
        
        int number = 0;
        try {
            number = Integer.valueOf(test).intValue();
        } catch (Exception ignore) {}
        
        return number;
    }
   
    /**
     * Reads the content of a file (fully)
     * @param file file to retrieve the content from
     * @return content of the file as a byte array
     * @throws Exception
     */
    public static byte[] readFile(File file) throws Exception {
        RandomAccessFile access = new RandomAccessFile(file, "rw");
        byte[] byties = new byte[new Long(access.length()).intValue()];
        access.readFully(byties);
        access.close();
        return byties;        
    }

    public static ImageIcon base64ToImage(String base64) {
        byte[] bytes = Base64.decode(base64.toCharArray());
        return new ImageIcon(bytes);
    }
    
    /**
     * @deprecated
     */
    public static void writeImageToFile(String base64, String filename) throws Exception {
        FileOutputStream fos = new FileOutputStream(filename);
        BufferedOutputStream bos = new BufferedOutputStream(fos);

        bos.write(Base64.decode(base64.toCharArray()));
        bos.flush();
        bos.close();
    }    
    
    public static void writeImageToFile(byte[] image, String filename) throws Exception {
        FileOutputStream fos = new FileOutputStream(filename);
        BufferedOutputStream bos = new BufferedOutputStream(fos);

        bos.write(image);
        bos.flush();
        bos.close();
    }   
    
    public static Image getScaledImage(byte[] bytes) {
        ImageIcon icon = new ImageIcon(bytes);
        int imageWidth = icon.getIconWidth();
        int imageHeight = icon.getIconHeight();
        int w = 190;
        int h = 145;

        double scaledRatio = (double) w / (double) h;
        double imageRatio = (double) imageWidth / (double) imageHeight;
        if (scaledRatio < imageRatio)
            h = (int) (w / imageRatio);
        else
            w = (int) (h * imageRatio);

        return icon.getImage().getScaledInstance(w, h, BufferedImage.SCALE_SMOOTH);
    }
    
    public static void writeScaledImageToFile(byte[] bytes, String targetFile) throws Exception {
        Image image = getScaledImage(bytes);
        BufferedImage bufferedImage = toBufferedImage(new ImageIcon(image));
        ImageIO.write(bufferedImage, "jpg", new File(targetFile));
        bufferedImage.flush();
    }
    
    public static String getHexColor(Color color) {
        String hexColor = "#" + Integer.toHexString(color.getRed());
        hexColor += Integer.toHexString(color.getGreen());
        hexColor += Integer.toHexString(color.getBlue()); 
        return hexColor.toUpperCase();
    }    
    
    public static ImageIcon getRatingIcon(Object value) {
        ImageIcon icon = null;
        try {
            int rating = ((Integer) value).intValue();
            switch (rating) {
                case 0:
                    icon = IconLibrary._icoRating0;
                    break;
                case 1:
                    icon = IconLibrary._icoRating1;
                    break;
                case 2:
                    icon = IconLibrary._icoRating2;
                    break;
                case 3:
                    icon = IconLibrary._icoRating3;
                    break;
                case 4:
                    icon = IconLibrary._icoRating4;
                    break;                    
                case 5:
                    icon = IconLibrary._icoRating5;
                    break;                    
            }
        } catch (Exception ignore) {}
        return icon;
    }    

    public static String getCurrentDirectory() throws Exception {
    	File fl = new File(".");
    	fl = fl.getCanonicalFile();
        return fl.toString();
    }
    
    /**
     * Gets the content of a file and converts it to a base64 string
     * @param url url of file
     * @return base64 content of the file
     */
    public static String fileToBase64String(URL url) {
        try {
            if (url != null) {
                File file = new File(url.getFile());
                byte[] b = Utilities.readFile(file);
                file = null;
                return String.valueOf(Base64.encode(b));
            }
        } catch (Exception exp) {
            Messages.add(DcResources.getText("msgCouldNotLoadPicture"), Messages._ERROR);
            Messages.add(exp);
        }
        return "";
    }
    
    public static BufferedImage toBufferedImage(ImageIcon icon) {
        // This code ensures that all the pixels in the image are loaded
        Image image = icon.getImage();
    
        // Create a buffered image with a format that's compatible with the screen
        BufferedImage bufferedImg = null;
        GraphicsEnvironment ge = GraphicsEnvironment.getLocalGraphicsEnvironment();
        try {
            // Create the buffered image
            GraphicsDevice gs = ge.getDefaultScreenDevice();
            GraphicsConfiguration gc = gs.getDefaultConfiguration();
            bufferedImg = gc.createCompatibleImage(image.getWidth(null), image.getHeight(null), Transparency.OPAQUE);
        } catch (HeadlessException e) {}
        
        bufferedImg = bufferedImg == null ? new BufferedImage(image.getWidth(null), image.getHeight(null), BufferedImage.TYPE_INT_RGB) : bufferedImg;

        // Copy image to buffered image
        Graphics g = bufferedImg.createGraphics();
    
        // Paint the image onto the buffered image
        g.drawImage(image, 0, 0, null);
        g.dispose();
        image.flush();
    
        return bufferedImg;
    }         
}
